/*     */ package com.google.code.yanf4j.nio.impl;
/*     */ 
/*     */ import com.google.code.yanf4j.buffer.IoBuffer;
/*     */ import com.google.code.yanf4j.core.EventType;
/*     */ import com.google.code.yanf4j.core.Handler;
/*     */ import com.google.code.yanf4j.core.WriteMessage;
/*     */ import com.google.code.yanf4j.core.impl.AbstractSession;
/*     */ import com.google.code.yanf4j.core.impl.FutureImpl;
/*     */ import com.google.code.yanf4j.nio.NioSessionConfig;
/*     */ import com.google.code.yanf4j.util.SelectorFactory;
/*     */ import java.io.IOException;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.nio.channels.CancelledKeyException;
/*     */ import java.nio.channels.ClosedChannelException;
/*     */ import java.nio.channels.SelectableChannel;
/*     */ import java.nio.channels.SelectionKey;
/*     */ import java.nio.channels.Selector;
/*     */ import java.nio.channels.SocketChannel;
/*     */ import java.nio.channels.WritableByteChannel;
/*     */ import java.util.Queue;
/*     */ import java.util.concurrent.atomic.AtomicReference;
/*     */ import java.util.concurrent.locks.ReentrantLock;
/*     */ import org.slf4j.Logger;
/*     */ 
/*     */ public abstract class AbstractNioSession extends AbstractSession implements com.google.code.yanf4j.nio.NioSession
/*     */ {
/*     */   protected SelectorManager selectorManager;
/*     */   protected SelectableChannel selectableChannel;
/*     */   
/*     */   public SelectableChannel channel()
/*     */   {
/*  32 */     return this.selectableChannel;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public AbstractNioSession(NioSessionConfig sessionConfig)
/*     */   {
/*  39 */     super(sessionConfig);
/*  40 */     this.selectorManager = sessionConfig.selectorManager;
/*  41 */     this.selectableChannel = sessionConfig.selectableChannel;
/*     */   }
/*     */   
/*     */   public final void enableRead(Selector selector) {
/*  45 */     SelectionKey key = this.selectableChannel.keyFor(selector);
/*  46 */     if ((key != null) && (key.isValid())) {
/*  47 */       interestRead(key);
/*     */     } else {
/*     */       try {
/*  50 */         this.selectableChannel.register(selector, 1, this);
/*     */       }
/*     */       catch (ClosedChannelException e) {}catch (CancelledKeyException e) {}
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private void interestRead(SelectionKey key)
/*     */   {
/*  61 */     if (key.attachment() == null) {
/*  62 */       key.attach(this);
/*     */     }
/*  64 */     key.interestOps(key.interestOps() | 0x1);
/*     */   }
/*     */   
/*     */   protected void start0()
/*     */   {
/*  69 */     registerSession();
/*     */   }
/*     */   
/*     */   public java.net.InetAddress getLocalAddress() {
/*  73 */     return ((SocketChannel)this.selectableChannel).socket().getLocalAddress();
/*     */   }
/*     */   
/*     */   protected abstract Object writeToChannel(WriteMessage paramWriteMessage) throws ClosedChannelException, IOException;
/*     */   
/*     */   protected void onWrite(SelectionKey key)
/*     */   {
/*  80 */     boolean isLockedByMe = false;
/*  81 */     if (this.currentMessage.get() == null)
/*     */     {
/*  83 */       WriteMessage nextMessage = (WriteMessage)this.writeQueue.peek();
/*  84 */       if ((nextMessage != null) && (this.writeLock.tryLock())) {
/*  85 */         if ((!this.writeQueue.isEmpty()) && (this.currentMessage.compareAndSet(null, nextMessage)))
/*     */         {
/*  87 */           this.writeQueue.remove();
/*     */         }
/*     */       } else {
/*  90 */         return;
/*     */       }
/*  92 */     } else if (!this.writeLock.tryLock()) {
/*  93 */       return;
/*     */     }
/*     */     
/*  96 */     isLockedByMe = true;
/*  97 */     WriteMessage currentMessage = null;
/*     */     
/*  99 */     long maxWritten = this.readBuffer.capacity() + this.readBuffer.capacity() >>> 1;
/*     */     try {
/* 101 */       long written = 0L;
/* 102 */       while (this.currentMessage.get() != null) {
/* 103 */         currentMessage = (WriteMessage)this.currentMessage.get();
/* 104 */         currentMessage = preprocessWriteMessage(currentMessage);
/* 105 */         this.currentMessage.set(currentMessage);
/* 106 */         long before = ((WriteMessage)this.currentMessage.get()).getWriteBuffer().remaining();
/*     */         
/* 108 */         Object writeResult = null;
/*     */         
/* 110 */         if (written < maxWritten) {
/* 111 */           writeResult = writeToChannel(currentMessage);
/* 112 */           written += ((WriteMessage)this.currentMessage.get()).getWriteBuffer().remaining() - before;
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 119 */         if (writeResult != null) {
/* 120 */           this.currentMessage.set(this.writeQueue.poll());
/* 121 */           this.handler.onMessageSent(this, currentMessage.getMessage());
/*     */           
/* 123 */           if (this.currentMessage.get() == null) {
/* 124 */             if (isLockedByMe) {
/* 125 */               isLockedByMe = false;
/* 126 */               this.writeLock.unlock();
/*     */             }
/*     */             
/* 129 */             WriteMessage nextMessage = (WriteMessage)this.writeQueue.peek();
/* 130 */             if ((nextMessage == null) || (!this.writeLock.tryLock())) break;
/* 131 */             isLockedByMe = true;
/* 132 */             if ((!this.writeQueue.isEmpty()) && (this.currentMessage.compareAndSet(null, nextMessage)))
/*     */             {
/*     */ 
/* 135 */               this.writeQueue.remove();
/*     */             }
/*     */             
/*     */           }
/*     */           
/*     */ 
/*     */         }
/*     */         else
/*     */         {
/* 144 */           if (isLockedByMe) {
/* 145 */             isLockedByMe = false;
/* 146 */             this.writeLock.unlock();
/*     */           }
/*     */           
/* 149 */           this.selectorManager.registerSession(this, EventType.ENABLE_WRITE);
/*     */           
/* 151 */           break;
/*     */         }
/*     */       }
/*     */     } catch (IOException e) {
/* 155 */       this.handler.onExceptionCaught(this, e);
/* 156 */       if ((currentMessage != null) && (currentMessage.getWriteFuture() != null))
/*     */       {
/* 158 */         currentMessage.getWriteFuture().failure(e);
/*     */       }
/* 160 */       if (isLockedByMe) {
/* 161 */         isLockedByMe = false;
/* 162 */         this.writeLock.unlock();
/*     */       }
/* 164 */       close();
/*     */     } finally {
/* 166 */       if (isLockedByMe) {
/* 167 */         this.writeLock.unlock();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public final void enableWrite(Selector selector) {
/* 173 */     SelectionKey key = this.selectableChannel.keyFor(selector);
/* 174 */     if ((key != null) && (key.isValid())) {
/* 175 */       interestWrite(key);
/*     */     } else {
/*     */       try {
/* 178 */         this.selectableChannel.register(selector, 4, this);
/*     */       }
/*     */       catch (ClosedChannelException e) {}catch (CancelledKeyException e) {}
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private void interestWrite(SelectionKey key)
/*     */   {
/* 189 */     if (key.attachment() == null) {
/* 190 */       key.attach(this);
/*     */     }
/* 192 */     key.interestOps(key.interestOps() | 0x4);
/*     */   }
/*     */   
/*     */   protected void onRead(SelectionKey key) {
/* 196 */     readFromBuffer();
/*     */   }
/*     */   
/*     */   protected abstract void readFromBuffer();
/*     */   
/*     */   protected void closeChannel() throws IOException
/*     */   {
/* 203 */     flush0();
/* 204 */     unregisterSession();
/* 205 */     unregisterChannel();
/*     */   }
/*     */   
/*     */   protected final void unregisterChannel() throws IOException {
/* 209 */     this.writeLock.lock();
/*     */     try {
/* 211 */       if (getAttribute(SelectorManager.REACTOR_ATTRIBUTE) != null) {
/* 212 */         ((Reactor)getAttribute(SelectorManager.REACTOR_ATTRIBUTE)).unregisterChannel(this.selectableChannel);
/*     */       }
/*     */       
/* 215 */       if (this.selectableChannel.isOpen()) {
/* 216 */         this.selectableChannel.close();
/*     */       }
/*     */     } finally {
/* 219 */       this.writeLock.unlock();
/*     */     }
/*     */   }
/*     */   
/*     */   protected final void registerSession() {
/* 224 */     this.selectorManager.registerSession(this, EventType.REGISTER);
/*     */   }
/*     */   
/*     */   protected void unregisterSession() {
/* 228 */     this.selectorManager.registerSession(this, EventType.UNREGISTER);
/*     */   }
/*     */   
/*     */   public void writeFromUserCode(WriteMessage message)
/*     */   {
/* 233 */     if (schduleWriteMessage(message)) {
/* 234 */       return;
/*     */     }
/*     */     
/* 237 */     onWrite(null);
/*     */   }
/*     */   
/*     */   protected boolean schduleWriteMessage(WriteMessage writeMessage)
/*     */   {
/* 242 */     boolean offered = this.writeQueue.offer(writeMessage);
/* 243 */     assert (offered);
/* 244 */     Reactor reactor = this.selectorManager.getReactorFromSession(this);
/* 245 */     if (Thread.currentThread() != reactor) {
/* 246 */       this.selectorManager.registerSession(this, EventType.ENABLE_WRITE);
/* 247 */       return true;
/*     */     }
/* 249 */     return false;
/*     */   }
/*     */   
/*     */   public void flush() {
/* 253 */     if (isClosed()) {
/* 254 */       return;
/*     */     }
/* 256 */     flush0();
/*     */   }
/*     */   
/*     */   protected final void flush0() {
/* 260 */     SelectionKey tmpKey = null;
/* 261 */     Selector writeSelector = null;
/* 262 */     int attempts = 0;
/*     */     try {
/*     */       do {
/* 265 */         if (writeSelector == null) {
/* 266 */           writeSelector = SelectorFactory.getSelector();
/* 267 */           if (writeSelector == null) {
/*     */             return;
/*     */           }
/* 270 */           tmpKey = this.selectableChannel.register(writeSelector, 4);
/*     */         }
/*     */         
/* 273 */         if (writeSelector.select(1000L) != 0) break;
/* 274 */         attempts++;
/* 275 */       } while (attempts <= 2);
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 282 */       return;onWrite(this.selectableChannel.keyFor(writeSelector));
/*     */     } catch (ClosedChannelException cce) {
/* 284 */       onException(cce);
/* 285 */       log.error("Flush error", cce);
/* 286 */       close();
/*     */     } catch (IOException ioe) {
/* 288 */       onException(ioe);
/* 289 */       log.error("Flush error", ioe);
/* 290 */       close();
/*     */     } finally {
/* 292 */       if (tmpKey != null)
/*     */       {
/* 294 */         tmpKey.cancel();
/* 295 */         tmpKey = null;
/*     */       }
/* 297 */       if (writeSelector != null) {
/*     */         try {
/* 299 */           writeSelector.selectNow();
/*     */         } catch (IOException e) {
/* 301 */           log.error("Temp selector selectNow error", e);
/*     */         }
/*     */         
/* 304 */         SelectorFactory.returnSelector(writeSelector);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   protected final long doRealWrite(SelectableChannel channel, IoBuffer buffer) throws IOException
/*     */   {
/* 311 */     if (log.isDebugEnabled()) {
/* 312 */       StringBuffer bufMsg = new StringBuffer("send buffers:\n[\n");
/* 313 */       ByteBuffer buff = buffer.buf();
/* 314 */       bufMsg.append(" buffer:position=").append(buff.position()).append(",limit=").append(buff.limit()).append(",capacity=").append(buff.capacity()).append("\n");
/*     */       
/*     */ 
/*     */ 
/* 318 */       bufMsg.append("]");
/* 319 */       log.debug(bufMsg.toString());
/*     */     }
/* 321 */     return ((WritableByteChannel)channel).write(buffer.buf());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public final void onEvent(EventType event, Selector selector)
/*     */   {
/* 328 */     if (isClosed()) {
/* 329 */       return;
/*     */     }
/* 331 */     SelectionKey key = this.selectableChannel.keyFor(selector);
/*     */     
/* 333 */     switch (event) {
/*     */     case EXPIRED: 
/* 335 */       onExpired();
/* 336 */       break;
/*     */     case WRITEABLE: 
/* 338 */       onWrite(key);
/* 339 */       break;
/*     */     case READABLE: 
/* 341 */       onRead(key);
/* 342 */       break;
/*     */     case ENABLE_WRITE: 
/* 344 */       enableWrite(selector);
/* 345 */       break;
/*     */     case ENABLE_READ: 
/* 347 */       enableRead(selector);
/* 348 */       break;
/*     */     case IDLE: 
/* 350 */       onIdle();
/* 351 */       break;
/*     */     case CONNECTED: 
/* 353 */       onConnected();
/* 354 */       break;
/*     */     default: 
/* 356 */       log.error("Unknown event:" + event.name());
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\nio\impl\AbstractNioSession.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */