/*     */ package com.google.code.yanf4j.statistics.impl;
/*     */ 
/*     */ import com.google.code.yanf4j.statistics.Statistics;
/*     */ import java.util.concurrent.atomic.AtomicLong;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SimpleStatistics
/*     */   implements Statistics
/*     */ {
/*  36 */   private boolean started = false;
/*     */   
/*     */   public boolean isStatistics() {
/*  39 */     return this.started;
/*     */   }
/*     */   
/*     */   public synchronized void reset() {
/*  43 */     if (this.started) {
/*  44 */       throw new IllegalStateException();
/*     */     }
/*  46 */     this.startTime = (this.stopTime = -1L);
/*  47 */     this.recvMessageCount.set(0L);
/*  48 */     this.recvMessageTotalSize.set(0L);
/*  49 */     this.writeMessageCount.set(0L);
/*  50 */     this.writeMessageTotalSize.set(0L);
/*  51 */     this.processMessageCount.set(0L);
/*  52 */     this.processMessageTotalTime.set(0L);
/*  53 */     this.acceptCount.set(0L);
/*     */   }
/*     */   
/*  56 */   private double receiveThroughputLimit = -1.0D;
/*  57 */   private double sendThroughputLimit = -1.0D;
/*     */   private long startTime;
/*     */   
/*  60 */   public void setReceiveThroughputLimit(double receivePacketRate) { this.receiveThroughputLimit = receivePacketRate; }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean isReceiveOverFlow()
/*     */   {
/*  69 */     if (getReceiveThroughputLimit() < 0.0D) {
/*  70 */       return false;
/*     */     }
/*  72 */     return getReceiveBytesPerSecond() > getReceiveThroughputLimit();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean isSendOverFlow()
/*     */   {
/*  81 */     if (getSendThroughputLimit() < 0.0D) {
/*  82 */       return false;
/*     */     }
/*  84 */     return getSendBytesPerSecond() > getSendThroughputLimit();
/*     */   }
/*     */   
/*     */   public double getSendThroughputLimit() {
/*  88 */     return this.sendThroughputLimit;
/*     */   }
/*     */   
/*     */   public void setSendThroughputLimit(double sendThroughputLimit) {
/*  92 */     this.sendThroughputLimit = sendThroughputLimit;
/*     */   }
/*     */   
/*     */   public final double getReceiveThroughputLimit() {
/*  96 */     return this.receiveThroughputLimit;
/*     */   }
/*     */   
/*     */   public synchronized void restart() {
/* 100 */     stop();
/* 101 */     reset();
/* 102 */     start();
/*     */   }
/*     */   
/* 105 */   private long stopTime = -1L;
/*     */   
/* 107 */   private AtomicLong recvMessageCount = new AtomicLong();
/*     */   
/* 109 */   private AtomicLong recvMessageTotalSize = new AtomicLong();
/*     */   
/* 111 */   private AtomicLong writeMessageCount = new AtomicLong();
/*     */   
/* 113 */   private AtomicLong writeMessageTotalSize = new AtomicLong();
/*     */   
/* 115 */   private AtomicLong processMessageCount = new AtomicLong();
/*     */   
/* 117 */   private AtomicLong acceptCount = new AtomicLong();
/*     */   
/* 119 */   private AtomicLong processMessageTotalTime = new AtomicLong();
/*     */   
/*     */   public long getStartedTime() {
/* 122 */     return this.startTime;
/*     */   }
/*     */   
/*     */   public double getProcessedMessageAverageTime() {
/* 126 */     return this.processMessageCount.get() == 0L ? 0.0D : this.processMessageTotalTime.get() / this.processMessageCount.get();
/*     */   }
/*     */   
/*     */ 
/*     */   public long getProcessedMessageCount()
/*     */   {
/* 132 */     return this.processMessageCount.get();
/*     */   }
/*     */   
/*     */   public void statisticsProcess(long n) {
/* 136 */     if (!this.started) {
/* 137 */       return;
/*     */     }
/* 139 */     if (n < 0L) {
/* 140 */       return;
/*     */     }
/* 142 */     this.processMessageTotalTime.addAndGet(n);
/* 143 */     this.processMessageCount.incrementAndGet();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public synchronized void start()
/*     */   {
/* 151 */     this.startTime = System.currentTimeMillis();
/* 152 */     this.started = true;
/*     */   }
/*     */   
/*     */   public synchronized void stop() {
/* 156 */     this.stopTime = System.currentTimeMillis();
/* 157 */     this.started = false;
/*     */   }
/*     */   
/*     */   public void statisticsRead(long n) {
/* 161 */     if (!this.started) {
/* 162 */       return;
/*     */     }
/* 164 */     if (n <= 0L) {
/* 165 */       return;
/*     */     }
/* 167 */     this.recvMessageCount.incrementAndGet();
/* 168 */     this.recvMessageTotalSize.addAndGet(n);
/*     */   }
/*     */   
/*     */   public long getRecvMessageCount() {
/* 172 */     return this.recvMessageCount.get();
/*     */   }
/*     */   
/*     */   public long getRecvMessageTotalSize() {
/* 176 */     return this.recvMessageTotalSize.get();
/*     */   }
/*     */   
/*     */   public long getWriteMessageCount() {
/* 180 */     return this.writeMessageCount.get();
/*     */   }
/*     */   
/*     */   public long getWriteMessageTotalSize() {
/* 184 */     return this.writeMessageTotalSize.get();
/*     */   }
/*     */   
/*     */   public void statisticsWrite(long n) {
/* 188 */     if (!this.started) {
/* 189 */       return;
/*     */     }
/* 191 */     if (n <= 0L) {
/* 192 */       return;
/*     */     }
/* 194 */     this.writeMessageCount.incrementAndGet();
/* 195 */     this.writeMessageTotalSize.addAndGet(n);
/*     */   }
/*     */   
/*     */   public long getRecvMessageAverageSize() {
/* 199 */     return this.recvMessageCount.get() == 0L ? 0L : this.recvMessageTotalSize.get() / this.recvMessageCount.get();
/*     */   }
/*     */   
/*     */ 
/*     */   public double getRecvMessageCountPerSecond()
/*     */   {
/* 205 */     long duration = this.stopTime == -1L ? System.currentTimeMillis() - this.startTime : this.stopTime - this.startTime;
/*     */     
/* 207 */     return duration == 0L ? 0.0D : this.recvMessageCount.get() * 1000.0D / duration;
/*     */   }
/*     */   
/*     */   public double getWriteMessageCountPerSecond()
/*     */   {
/* 212 */     long duration = this.stopTime == -1L ? System.currentTimeMillis() - this.startTime : this.stopTime - this.startTime;
/*     */     
/* 214 */     return duration == 0L ? 0.0D : this.writeMessageCount.get() * 1000.0D / duration;
/*     */   }
/*     */   
/*     */   public long getWriteMessageAverageSize()
/*     */   {
/* 219 */     return this.writeMessageCount.get() == 0L ? 0L : this.writeMessageTotalSize.get() / this.writeMessageCount.get();
/*     */   }
/*     */   
/*     */ 
/*     */   public double getAcceptCountPerSecond()
/*     */   {
/* 225 */     long duration = this.stopTime == -1L ? System.currentTimeMillis() - this.startTime : this.stopTime - this.startTime;
/*     */     
/* 227 */     return duration == 0L ? 0.0D : this.acceptCount.get() * 1000.0D / duration;
/*     */   }
/*     */   
/*     */   public double getReceiveBytesPerSecond()
/*     */   {
/* 232 */     long duration = this.stopTime == -1L ? System.currentTimeMillis() - this.startTime : this.stopTime - this.startTime;
/*     */     
/* 234 */     return duration == 0L ? 0.0D : this.recvMessageTotalSize.get() * 1000.0D / duration;
/*     */   }
/*     */   
/*     */   public double getSendBytesPerSecond()
/*     */   {
/* 239 */     long duration = this.stopTime == -1L ? System.currentTimeMillis() - this.startTime : this.stopTime - this.startTime;
/*     */     
/* 241 */     return duration == 0L ? 0.0D : this.writeMessageTotalSize.get() * 1000.0D / duration;
/*     */   }
/*     */   
/*     */   public void statisticsAccept()
/*     */   {
/* 246 */     if (!this.started) {
/* 247 */       return;
/*     */     }
/* 249 */     this.acceptCount.incrementAndGet();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\statistics\impl\SimpleStatistics.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */