/*     */ package com.google.code.yanf4j.util;
/*     */ 
/*     */ import java.io.Serializable;
/*     */ import java.util.AbstractList;
/*     */ import java.util.Arrays;
/*     */ import java.util.List;
/*     */ import java.util.NoSuchElementException;
/*     */ import java.util.Queue;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class CircularQueue<E>
/*     */   extends AbstractList<E>
/*     */   implements List<E>, Queue<E>, Serializable
/*     */ {
/*     */   private static final long serialVersionUID = 3993421269224511264L;
/*     */   private static final int DEFAULT_CAPACITY = 4;
/*     */   private final int initialCapacity;
/*     */   private volatile Object[] items;
/*     */   private int mask;
/*  52 */   private int first = 0;
/*  53 */   private int last = 0;
/*     */   
/*     */   private boolean full;
/*     */   
/*     */   private int shrinkThreshold;
/*     */   
/*     */   public CircularQueue()
/*     */   {
/*  61 */     this(4);
/*     */   }
/*     */   
/*     */   public CircularQueue(int initialCapacity) {
/*  65 */     int actualCapacity = normalizeCapacity(initialCapacity);
/*  66 */     this.items = new Object[actualCapacity];
/*  67 */     this.mask = (actualCapacity - 1);
/*  68 */     this.initialCapacity = actualCapacity;
/*  69 */     this.shrinkThreshold = 0;
/*     */   }
/*     */   
/*     */   private static int normalizeCapacity(int initialCapacity) {
/*  73 */     int actualCapacity = 1;
/*  74 */     while (actualCapacity < initialCapacity) {
/*  75 */       actualCapacity <<= 1;
/*  76 */       if (actualCapacity < 0) {
/*  77 */         actualCapacity = 1073741824;
/*     */       }
/*     */     }
/*     */     
/*  81 */     return actualCapacity;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public int capacity()
/*     */   {
/*  88 */     return this.items.length;
/*     */   }
/*     */   
/*     */   public void clear()
/*     */   {
/*  93 */     if (!isEmpty()) {
/*  94 */       Arrays.fill(this.items, null);
/*  95 */       this.first = 0;
/*  96 */       this.last = 0;
/*  97 */       this.full = false;
/*  98 */       shrinkIfNeeded();
/*     */     }
/*     */   }
/*     */   
/*     */   public E poll()
/*     */   {
/* 104 */     if (isEmpty()) {
/* 105 */       return null;
/*     */     }
/*     */     
/* 108 */     Object ret = this.items[this.first];
/* 109 */     this.items[this.first] = null;
/* 110 */     decreaseSize();
/*     */     
/* 112 */     if (this.first == this.last) {
/* 113 */       this.first = (this.last = 0);
/*     */     }
/*     */     
/* 116 */     shrinkIfNeeded();
/* 117 */     return (E)ret;
/*     */   }
/*     */   
/*     */   public boolean offer(E item) {
/* 121 */     if (item == null) {
/* 122 */       throw new NullPointerException("item");
/*     */     }
/* 124 */     expandIfNeeded();
/* 125 */     this.items[this.last] = item;
/* 126 */     increaseSize();
/* 127 */     return true;
/*     */   }
/*     */   
/*     */   public E peek()
/*     */   {
/* 132 */     if (isEmpty()) {
/* 133 */       return null;
/*     */     }
/*     */     
/* 136 */     return (E)this.items[this.first];
/*     */   }
/*     */   
/*     */ 
/*     */   public E get(int idx)
/*     */   {
/* 142 */     checkIndex(idx);
/* 143 */     return (E)this.items[getRealIndex(idx)];
/*     */   }
/*     */   
/*     */   public boolean isEmpty()
/*     */   {
/* 148 */     return (this.first == this.last) && (!this.full);
/*     */   }
/*     */   
/*     */   public int size()
/*     */   {
/* 153 */     if (this.full) {
/* 154 */       return capacity();
/*     */     }
/*     */     
/* 157 */     if (this.last >= this.first) {
/* 158 */       return this.last - this.first;
/*     */     }
/* 160 */     return this.last - this.first + capacity();
/*     */   }
/*     */   
/*     */ 
/*     */   public String toString()
/*     */   {
/* 166 */     return "first=" + this.first + ", last=" + this.last + ", size=" + size() + ", mask = " + this.mask;
/*     */   }
/*     */   
/*     */   private void checkIndex(int idx)
/*     */   {
/* 171 */     if ((idx < 0) || (idx >= size())) {
/* 172 */       throw new IndexOutOfBoundsException(String.valueOf(idx));
/*     */     }
/*     */   }
/*     */   
/*     */   private int getRealIndex(int idx) {
/* 177 */     return this.first + idx & this.mask;
/*     */   }
/*     */   
/*     */   private void increaseSize() {
/* 181 */     this.last = (this.last + 1 & this.mask);
/* 182 */     this.full = (this.first == this.last);
/*     */   }
/*     */   
/*     */   private void decreaseSize() {
/* 186 */     this.first = (this.first + 1 & this.mask);
/* 187 */     this.full = false;
/*     */   }
/*     */   
/*     */   private void expandIfNeeded() {
/* 191 */     if (this.full)
/*     */     {
/* 193 */       int oldLen = this.items.length;
/* 194 */       int newLen = oldLen << 1;
/* 195 */       Object[] tmp = new Object[newLen];
/*     */       
/* 197 */       if (this.first < this.last) {
/* 198 */         System.arraycopy(this.items, this.first, tmp, 0, this.last - this.first);
/*     */       }
/*     */       else {
/* 201 */         System.arraycopy(this.items, this.first, tmp, 0, oldLen - this.first);
/*     */         
/* 203 */         System.arraycopy(this.items, 0, tmp, oldLen - this.first, this.last);
/*     */       }
/*     */       
/*     */ 
/* 207 */       this.first = 0;
/* 208 */       this.last = oldLen;
/* 209 */       this.items = tmp;
/* 210 */       this.mask = (tmp.length - 1);
/* 211 */       if (newLen >>> 3 > this.initialCapacity) {
/* 212 */         this.shrinkThreshold = (newLen >>> 3);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private void shrinkIfNeeded() {
/* 218 */     int size = size();
/* 219 */     if (size <= this.shrinkThreshold)
/*     */     {
/* 221 */       int oldLen = this.items.length;
/* 222 */       int newLen = normalizeCapacity(size);
/* 223 */       if (size == newLen) {
/* 224 */         newLen <<= 1;
/*     */       }
/*     */       
/* 227 */       if (newLen >= oldLen) {
/* 228 */         return;
/*     */       }
/*     */       
/* 231 */       if (newLen < this.initialCapacity) {
/* 232 */         if (oldLen == this.initialCapacity) {
/* 233 */           return;
/*     */         }
/* 235 */         newLen = this.initialCapacity;
/*     */       }
/*     */       
/*     */ 
/* 239 */       Object[] tmp = new Object[newLen];
/*     */       
/*     */ 
/* 242 */       if (size > 0) {
/* 243 */         if (this.first < this.last) {
/* 244 */           System.arraycopy(this.items, this.first, tmp, 0, this.last - this.first);
/*     */         }
/*     */         else {
/* 247 */           System.arraycopy(this.items, this.first, tmp, 0, oldLen - this.first);
/*     */           
/* 249 */           System.arraycopy(this.items, 0, tmp, oldLen - this.first, this.last);
/*     */         }
/*     */       }
/*     */       
/*     */ 
/* 254 */       this.first = 0;
/* 255 */       this.last = size;
/* 256 */       this.items = tmp;
/* 257 */       this.mask = (tmp.length - 1);
/* 258 */       this.shrinkThreshold = 0;
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean add(E o)
/*     */   {
/* 264 */     return offer(o);
/*     */   }
/*     */   
/*     */ 
/*     */   public E set(int idx, E o)
/*     */   {
/* 270 */     checkIndex(idx);
/*     */     
/* 272 */     int realIdx = getRealIndex(idx);
/* 273 */     Object old = this.items[realIdx];
/* 274 */     this.items[realIdx] = o;
/* 275 */     return (E)old;
/*     */   }
/*     */   
/*     */   public void add(int idx, E o)
/*     */   {
/* 280 */     if (idx == size()) {
/* 281 */       offer(o);
/* 282 */       return;
/*     */     }
/*     */     
/* 285 */     checkIndex(idx);
/* 286 */     expandIfNeeded();
/*     */     
/* 288 */     int realIdx = getRealIndex(idx);
/*     */     
/*     */ 
/* 291 */     if (this.first < this.last) {
/* 292 */       System.arraycopy(this.items, realIdx, this.items, realIdx + 1, this.last - realIdx);
/*     */ 
/*     */     }
/* 295 */     else if (realIdx >= this.first) {
/* 296 */       System.arraycopy(this.items, 0, this.items, 1, this.last);
/* 297 */       this.items[0] = this.items[(this.items.length - 1)];
/* 298 */       System.arraycopy(this.items, realIdx, this.items, realIdx + 1, this.items.length - realIdx - 1);
/*     */     }
/*     */     else {
/* 301 */       System.arraycopy(this.items, realIdx, this.items, realIdx + 1, this.last - realIdx);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 306 */     this.items[realIdx] = o;
/* 307 */     increaseSize();
/*     */   }
/*     */   
/*     */ 
/*     */   public E remove(int idx)
/*     */   {
/* 313 */     if (idx == 0) {
/* 314 */       return (E)poll();
/*     */     }
/*     */     
/* 317 */     checkIndex(idx);
/*     */     
/* 319 */     int realIdx = getRealIndex(idx);
/* 320 */     Object removed = this.items[realIdx];
/*     */     
/*     */ 
/* 323 */     if (this.first < this.last) {
/* 324 */       System.arraycopy(this.items, this.first, this.items, this.first + 1, realIdx - this.first);
/*     */ 
/*     */     }
/* 327 */     else if (realIdx >= this.first) {
/* 328 */       System.arraycopy(this.items, this.first, this.items, this.first + 1, realIdx - this.first);
/*     */     }
/*     */     else {
/* 331 */       System.arraycopy(this.items, 0, this.items, 1, realIdx);
/* 332 */       this.items[0] = this.items[(this.items.length - 1)];
/* 333 */       System.arraycopy(this.items, this.first, this.items, this.first + 1, this.items.length - this.first - 1);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 338 */     this.items[this.first] = null;
/* 339 */     decreaseSize();
/*     */     
/* 341 */     shrinkIfNeeded();
/* 342 */     return (E)removed;
/*     */   }
/*     */   
/*     */   public E remove() {
/* 346 */     if (isEmpty()) {
/* 347 */       throw new NoSuchElementException();
/*     */     }
/* 349 */     return (E)poll();
/*     */   }
/*     */   
/*     */   public E element() {
/* 353 */     if (isEmpty()) {
/* 354 */       throw new NoSuchElementException();
/*     */     }
/* 356 */     return (E)peek();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\util\CircularQueue.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */