/*     */ package com.netapp.asup.config;
/*     */ 
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.exception.AsupCacheException;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.net.URL;
/*     */ import java.util.Map;
/*     */ import java.util.Properties;
/*     */ import java.util.concurrent.ConcurrentHashMap;
/*     */ import java.util.concurrent.ConcurrentMap;
/*     */ 
/*     */ 
/*     */ public class ConfigurationImpl
/*     */   implements Configuration
/*     */ {
/*     */   private static final String CACHE_CONFIG_PROPERTIES = "conf/asup-cache.properties";
/*  18 */   private static final AsupLogger LOG = AsupLogger.getLogger(ConfigurationImpl.class);
/*     */   
/*  20 */   private static final ConcurrentMap<ClassLoader, Map<String, Class<?>>> CACHE_CLASSES = new ConcurrentHashMap();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private Properties properties;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private ClassLoader classLoader;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ConfigurationImpl()
/*     */     throws AsupCacheException, IOException
/*     */   {
/*  58 */     this.classLoader = Thread.currentThread().getContextClassLoader();
/*  59 */     if (this.classLoader == null) {
/*  60 */       this.classLoader = ConfigurationImpl.class.getClassLoader();
/*     */     }
/*  25 */     this.properties = new Properties();
/*  26 */     loadDefaultResource("conf/asup-cache.properties");
/*     */   }
/*     */   
/*     */   public void loadDefaultResource(String resourceName)
/*     */     throws AsupCacheException
/*     */   {
/*  32 */     InputStream resourceAsStream = getClass().getClassLoader().getResourceAsStream(resourceName);
/*  33 */     if (null == resourceAsStream) {
/*  34 */       LOG.warn("The default configuration is not avaialble: " + resourceName, "");
/*  35 */       return;
/*     */     }
/*     */     try {
/*  38 */       this.properties.load(resourceAsStream);
/*     */     } catch (IOException e) {
/*  40 */       throw new AsupCacheException(LOG, "", "Unable to load the default configuration file:" + resourceName);
/*     */     }
/*     */   }
/*     */   
/*     */   public void loadResource(InputStream resourceAsStream) throws AsupCacheException
/*     */   {
/*  46 */     if (null == resourceAsStream) {
/*  47 */       throw new AsupCacheException(LOG, "", "Resource stream cannot be null:");
/*     */     }
/*     */     try {
/*  50 */       this.properties.load(resourceAsStream);
/*     */     } catch (IOException e) {
/*  52 */       throw new AsupCacheException(LOG, "", "Error while loading the property resource.", e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public URL getResource(String name)
/*     */   {
/*  65 */     return this.classLoader.getResource(name);
/*     */   }
/*     */   
/*     */   public String getTrimmed(String name) {
/*  69 */     if (null == name) {
/*  70 */       return null;
/*     */     }
/*  72 */     String value = get(name);
/*     */     
/*  74 */     if (null == value) {
/*  75 */       return null;
/*     */     }
/*  77 */     return value.trim();
/*     */   }
/*     */   
/*     */   public String get(String name)
/*     */   {
/*  82 */     return this.properties.getProperty(name);
/*     */   }
/*     */   
/*     */   public Properties getAllProperties() {
/*  86 */     return this.properties;
/*     */   }
/*     */   
/*     */   public void set(String name, String value) {
/*  90 */     if (isEmpty(name)) {
/*  91 */       LOG.warn("Invalid configuration key is provided:" + name, "");
/*  92 */       return;
/*     */     }
/*  94 */     if (value == null) {
/*  95 */       LOG.warn("Invalid configuration value is provided:" + value, "");
/*  96 */       return;
/*     */     }
/*  98 */     this.properties.setProperty(name, value);
/*     */   }
/*     */   
/*     */   public void setString(String name, String... values) {
/* 102 */     set(name, arrayToString(values));
/*     */   }
/*     */   
/*     */   public void setInt(String name, int value) {
/* 106 */     set(name, Integer.toString(value));
/*     */   }
/*     */   
/*     */   public void setDouble(String name, double value) {
/* 110 */     set(name, Double.toString(value));
/*     */   }
/*     */   
/*     */   public static String arrayToString(String[] strs) {
/* 114 */     if (null == strs) {
/* 115 */       return null;
/*     */     }
/* 117 */     if (strs.length == 0) {
/* 118 */       return "";
/*     */     }
/* 120 */     StringBuffer sbuf = new StringBuffer();
/* 121 */     sbuf.append(strs[0]);
/* 122 */     for (int idx = 1; idx < strs.length; idx++) {
/* 123 */       sbuf.append(",");
/* 124 */       sbuf.append(strs[idx]);
/*     */     }
/* 126 */     return sbuf.toString();
/*     */   }
/*     */   
/*     */   public String getString(String name) {
/* 130 */     if (isEmpty(name)) {
/* 131 */       LOG.warn("Invalid configuration key is provided:" + name, "");
/* 132 */       return null;
/*     */     }
/* 134 */     return this.properties.getProperty(name);
/*     */   }
/*     */   
/*     */   private boolean isEmpty(String name) {
/* 138 */     if ((null == name) || (name.length() <= 0)) {
/* 139 */       return true;
/*     */     }
/* 141 */     return false;
/*     */   }
/*     */   
/*     */   public int getInt(String name, int defaultValue) {
/* 145 */     String valueString = getTrimmed(name);
/* 146 */     if (valueString == null)
/* 147 */       return defaultValue;
/*     */     try {
/* 149 */       return Integer.parseInt(valueString);
/*     */     } catch (NumberFormatException e) {}
/* 151 */     return defaultValue;
/*     */   }
/*     */   
/*     */   public String getString(String name, String defaultValue)
/*     */   {
/* 156 */     String valueString = getTrimmed(name);
/* 157 */     if (valueString != null) {
/* 158 */       return valueString;
/*     */     }
/* 160 */     return defaultValue;
/*     */   }
/*     */   
/*     */   public double getDouble(String name, double defaultValue) {
/* 164 */     String valueString = getTrimmed(name);
/* 165 */     if (valueString == null)
/* 166 */       return defaultValue;
/*     */     try {
/* 168 */       return Double.parseDouble(valueString);
/*     */     } catch (NumberFormatException e) {}
/* 170 */     return defaultValue;
/*     */   }
/*     */   
/*     */   public Class<?> getClass(String name, Class<?> defaultValue)
/*     */   {
/* 175 */     String valueString = getTrimmed(name);
/* 176 */     if (valueString == null)
/* 177 */       return defaultValue;
/*     */     try {
/* 179 */       return getClassByName(valueString);
/*     */     } catch (ClassNotFoundException e) {
/* 181 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public Class<?> getClassByName(String name) throws ClassNotFoundException {
/* 186 */     Map<String, Class<?>> map = (Map)CACHE_CLASSES.get(this.classLoader);
/* 187 */     if (map == null) {
/* 188 */       Map<String, Class<?>> newMap = new ConcurrentHashMap();
/* 189 */       map = (Map)CACHE_CLASSES.putIfAbsent(this.classLoader, newMap);
/* 190 */       if (map == null) {
/* 191 */         map = newMap;
/*     */       }
/*     */     }
/*     */     
/* 195 */     Class<?> clazz = (Class)map.get(name);
/* 196 */     if (clazz == null) {
/* 197 */       clazz = Class.forName(name, true, this.classLoader);
/* 198 */       if (clazz != null) {
/* 199 */         map.put(name, clazz);
/*     */       }
/*     */     }
/*     */     
/* 203 */     return clazz;
/*     */   }
/*     */   
/*     */   public void setClass(String name, Class<?> theClass) {
/* 207 */     if (null == theClass) {
/* 208 */       LOG.warn("Invalid class value:" + theClass, "");
/* 209 */       return;
/*     */     }
/* 211 */     set(name, theClass.getName());
/*     */   }
/*     */   
/*     */   public Properties getProperties() {
/* 215 */     return this.properties;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\config\ConfigurationImpl.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */