/*     */ package net.rubyeye.xmemcached.transcoders;
/*     */ 
/*     */ import java.util.Date;
/*     */ import org.slf4j.Logger;
/*     */ 
/*     */ 
/*     */ 
/*     */ public class WhalinTranscoder
/*     */   extends BaseSerializingTranscoder
/*     */   implements Transcoder<Object>
/*     */ {
/*     */   public static final int SPECIAL_BYTE = 1;
/*     */   public static final int SPECIAL_BOOLEAN = 8192;
/*     */   public static final int SPECIAL_INT = 4;
/*     */   public static final int SPECIAL_LONG = 16384;
/*     */   public static final int SPECIAL_CHARACTER = 16;
/*     */   public static final int SPECIAL_STRING = 32;
/*     */   public static final int SPECIAL_STRINGBUFFER = 64;
/*     */   public static final int SPECIAL_FLOAT = 128;
/*     */   public static final int SPECIAL_SHORT = 256;
/*     */   public static final int SPECIAL_DOUBLE = 512;
/*     */   public static final int SPECIAL_DATE = 1024;
/*     */   public static final int SPECIAL_STRINGBUILDER = 2048;
/*     */   public static final int SPECIAL_BYTEARRAY = 4096;
/*     */   public static final int COMPRESSED = 2;
/*     */   public static final int SERIALIZED = 8;
/*     */   private int maxSize;
/*     */   private boolean primitiveAsString;
/*     */   
/*     */   public void setPackZeros(boolean packZeros)
/*     */   {
/*  32 */     this.tu.setPackZeros(packZeros);
/*     */   }
/*     */   
/*     */   public void setPrimitiveAsString(boolean primitiveAsString) {
/*  36 */     this.primitiveAsString = primitiveAsString;
/*     */   }
/*     */   
/*     */   public WhalinTranscoder() {
/*  40 */     this(CachedData.MAX_SIZE);
/*     */   }
/*     */   
/*     */   public WhalinTranscoder(int maxSize)
/*     */   {
/*  45 */     this.maxSize = maxSize;
/*     */   }
/*     */   
/*     */   public final int getMaxSize() {
/*  49 */     return this.maxSize;
/*     */   }
/*     */   
/*     */   public final void setMaxSize(int maxSize) {
/*  53 */     this.maxSize = maxSize;
/*     */   }
/*     */   
/*  56 */   public boolean isPackZeros() { return this.tu.isPackZeros(); }
/*     */   
/*     */   public boolean isPrimitiveAsString()
/*     */   {
/*  60 */     return this.primitiveAsString;
/*     */   }
/*     */   
/*  63 */   private final TranscoderUtils tu = new TranscoderUtils(false);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Object decode(CachedData d)
/*     */   {
/*  71 */     byte[] data = d.getData();
/*  72 */     Object rv = null;
/*  73 */     if ((d.getFlag() & 0x2) != 0) {
/*  74 */       data = decompress(d.getData());
/*     */     }
/*  76 */     if ((d.getFlag() & 0x8) != 0) {
/*  77 */       rv = deserialize(data);
/*     */     } else {
/*  79 */       int f = d.getFlag() & 0xFFFFFFFD;
/*  80 */       if ((this.primitiveAsString) && 
/*  81 */         (f == 32)) {
/*  82 */         return decodeString(d.getData());
/*     */       }
/*     */       
/*  85 */       switch (f) {
/*     */       case 8192: 
/*  87 */         rv = Boolean.valueOf(decodeBoolean(data));
/*  88 */         break;
/*     */       case 4: 
/*  90 */         rv = Integer.valueOf(this.tu.decodeInt(data));
/*  91 */         break;
/*     */       case 256: 
/*  93 */         rv = Short.valueOf((short)this.tu.decodeInt(data));
/*  94 */         break;
/*     */       case 16384: 
/*  96 */         rv = Long.valueOf(this.tu.decodeLong(data));
/*  97 */         break;
/*     */       case 1024: 
/*  99 */         rv = new Date(this.tu.decodeLong(data));
/* 100 */         break;
/*     */       case 1: 
/* 102 */         rv = Byte.valueOf(this.tu.decodeByte(data));
/* 103 */         break;
/*     */       case 128: 
/* 105 */         rv = new Float(Float.intBitsToFloat(this.tu.decodeInt(data)));
/* 106 */         break;
/*     */       case 512: 
/* 108 */         rv = new Double(Double.longBitsToDouble(this.tu.decodeLong(data)));
/*     */         
/* 110 */         break;
/*     */       case 4096: 
/* 112 */         rv = data;
/* 113 */         break;
/*     */       case 32: 
/* 115 */         rv = decodeString(data);
/* 116 */         break;
/*     */       case 64: 
/* 118 */         rv = new StringBuffer(decodeString(data));
/* 119 */         break;
/*     */       case 2048: 
/* 121 */         rv = new StringBuilder(decodeString(data));
/* 122 */         break;
/*     */       case 16: 
/* 124 */         rv = decodeCharacter(data);
/* 125 */         break;
/*     */       default: 
/* 127 */         log.warn(String.format("Cannot handle data with flags %x", new Object[] { Integer.valueOf(f) }));
/*     */       }
/*     */     }
/* 130 */     return rv;
/*     */   }
/*     */   
/*     */   public CachedData encode(Object o) {
/* 134 */     byte[] b = null;
/* 135 */     int flags = 0;
/* 136 */     if ((o instanceof String)) {
/* 137 */       b = encodeString((String)o);
/* 138 */       flags |= 0x20;
/* 139 */     } else if ((o instanceof StringBuffer)) {
/* 140 */       flags |= 0x40;
/* 141 */       b = encodeString(String.valueOf(o));
/* 142 */     } else if ((o instanceof StringBuilder)) {
/* 143 */       flags |= 0x800;
/* 144 */       b = encodeString(String.valueOf(o));
/* 145 */     } else if ((o instanceof Long)) {
/* 146 */       if (this.primitiveAsString) {
/* 147 */         b = encodeString(o.toString());
/*     */       } else {
/* 149 */         b = this.tu.encodeLong(((Long)o).longValue());
/*     */       }
/* 151 */       flags |= 0x4000;
/* 152 */     } else if ((o instanceof Integer)) {
/* 153 */       if (this.primitiveAsString) {
/* 154 */         b = encodeString(o.toString());
/*     */       } else {
/* 156 */         b = this.tu.encodeInt(((Integer)o).intValue());
/*     */       }
/* 158 */       flags |= 0x4;
/* 159 */     } else if ((o instanceof Short)) {
/* 160 */       if (this.primitiveAsString) {
/* 161 */         b = encodeString(o.toString());
/*     */       } else {
/* 163 */         b = this.tu.encodeInt(((Short)o).shortValue());
/*     */       }
/* 165 */       flags |= 0x100;
/* 166 */     } else if ((o instanceof Boolean)) {
/* 167 */       if (this.primitiveAsString) {
/* 168 */         b = encodeString(o.toString());
/*     */       } else {
/* 170 */         b = encodeBoolean(((Boolean)o).booleanValue());
/*     */       }
/* 172 */       flags |= 0x2000;
/* 173 */     } else if ((o instanceof Date)) {
/* 174 */       b = this.tu.encodeLong(((Date)o).getTime());
/* 175 */       flags |= 0x400;
/* 176 */     } else if ((o instanceof Byte)) {
/* 177 */       if (this.primitiveAsString) {
/* 178 */         b = encodeString(o.toString());
/*     */       } else {
/* 180 */         b = this.tu.encodeByte(((Byte)o).byteValue());
/*     */       }
/* 182 */       flags |= 0x1;
/* 183 */     } else if ((o instanceof Float)) {
/* 184 */       if (this.primitiveAsString) {
/* 185 */         b = encodeString(o.toString());
/*     */       } else {
/* 187 */         b = this.tu.encodeInt(Float.floatToIntBits(((Float)o).floatValue()));
/*     */       }
/* 189 */       flags |= 0x80;
/* 190 */     } else if ((o instanceof Double)) {
/* 191 */       if (this.primitiveAsString) {
/* 192 */         b = encodeString(o.toString());
/*     */       } else {
/* 194 */         b = this.tu.encodeLong(Double.doubleToLongBits(((Double)o).doubleValue()));
/*     */       }
/* 196 */       flags |= 0x200;
/* 197 */     } else if ((o instanceof byte[])) {
/* 198 */       b = (byte[])o;
/* 199 */       flags |= 0x1000;
/* 200 */     } else if ((o instanceof Character)) {
/* 201 */       b = this.tu.encodeInt(((Character)o).charValue());
/* 202 */       flags |= 0x10;
/*     */     } else {
/* 204 */       b = serialize(o);
/* 205 */       flags |= 0x8;
/*     */     }
/* 207 */     assert (b != null);
/* 208 */     if ((this.primitiveAsString) && 
/* 209 */       ((flags & 0x8) == 0)) {
/* 210 */       flags = 0;
/* 211 */       flags |= 0x20;
/*     */     }
/*     */     
/* 214 */     if (b.length > this.compressionThreshold) {
/* 215 */       byte[] compressed = compress(b);
/* 216 */       if (compressed.length < b.length) {
/* 217 */         if (log.isDebugEnabled()) {
/* 218 */           log.debug(String.format("Compressed %s from %d to %d", new Object[] { o.getClass().getName(), Integer.valueOf(b.length), Integer.valueOf(compressed.length) }));
/*     */         }
/*     */         
/*     */ 
/*     */ 
/* 223 */         b = compressed;
/* 224 */         flags |= 0x2;
/*     */       }
/* 226 */       else if (log.isDebugEnabled()) {
/* 227 */         log.debug(String.format("Compression increased the size of %s from %d to %d", new Object[] { o.getClass().getName(), Integer.valueOf(b.length), Integer.valueOf(compressed.length) }));
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 236 */     return new CachedData(flags, b);
/*     */   }
/*     */   
/*     */   protected Character decodeCharacter(byte[] b) {
/* 240 */     return Character.valueOf((char)this.tu.decodeInt(b));
/*     */   }
/*     */   
/*     */   public byte[] encodeBoolean(boolean b) {
/* 244 */     byte[] rv = new byte[1];
/* 245 */     rv[0] = ((byte)(b ? 1 : 0));
/* 246 */     return rv;
/*     */   }
/*     */   
/*     */   public boolean decodeBoolean(byte[] in) {
/* 250 */     assert (in.length == 1) : "Wrong length for a boolean";
/* 251 */     return in[0] == 1;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\transcoders\WhalinTranscoder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */