/*     */ package org.apache.avro;
/*     */ 
/*     */ import java.io.ByteArrayInputStream;
/*     */ import java.io.File;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.io.PrintStream;
/*     */ import java.io.StringWriter;
/*     */ import java.security.MessageDigest;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.Collections;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.LinkedHashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.Set;
/*     */ import org.codehaus.jackson.JsonFactory;
/*     */ import org.codehaus.jackson.JsonGenerator;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.map.ObjectMapper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class Protocol
/*     */ {
/*     */   public static final long VERSION = 1L;
/*  66 */   private static final Set<String> MESSAGE_RESERVED = new HashSet();
/*     */   
/*     */   private String name;
/*     */   private String namespace;
/*     */   private String doc;
/*     */   
/*     */   public class Message
/*     */   {
/*     */     private String name;
/*     */     private String doc;
/*     */     private Schema request;
/*  77 */     private final Schema.Props props = new Schema.Props(Protocol.MESSAGE_RESERVED);
/*     */     
/*     */ 
/*     */     private Message(String name, Map<String, String> doc, Schema propMap)
/*     */     {
/*  82 */       this.name = name;
/*  83 */       this.doc = doc;
/*  84 */       this.request = request;
/*     */       
/*  86 */       if (propMap != null)
/*  87 */         for (Map.Entry<String, String> prop : propMap.entrySet()) {
/*  88 */           addProp((String)prop.getKey(), (String)prop.getValue());
/*     */         }
/*     */     }
/*     */     
/*  92 */     public String getName() { return this.name; }
/*     */     
/*  94 */     public Schema getRequest() { return this.request; }
/*     */     
/*  96 */     public Schema getResponse() { return Schema.create(Schema.Type.NULL); }
/*     */     
/*     */     public Schema getErrors() {
/*  99 */       return Schema.createUnion(new ArrayList());
/*     */     }
/*     */     
/*     */     public boolean isOneWay() {
/* 103 */       return true;
/*     */     }
/*     */     
/* 106 */     public synchronized String getProp(String name) { return (String)this.props.get(name); }
/*     */     
/*     */     public synchronized void addProp(String name, String value) {
/* 109 */       this.props.add(name, value);
/*     */     }
/*     */     
/*     */     public Map<String, String> getProps() {
/* 113 */       return Collections.unmodifiableMap(this.props);
/*     */     }
/*     */     
/*     */     public String toString() {
/*     */       try {
/* 118 */         StringWriter writer = new StringWriter();
/* 119 */         JsonGenerator gen = Schema.FACTORY.createJsonGenerator(writer);
/* 120 */         toJson(gen);
/* 121 */         gen.flush();
/* 122 */         return writer.toString();
/*     */       } catch (IOException e) {
/* 124 */         throw new AvroRuntimeException(e);
/*     */       }
/*     */     }
/*     */     
/* 128 */     void toJson(JsonGenerator gen) throws IOException { gen.writeStartObject();
/* 129 */       if (this.doc != null) gen.writeStringField("doc", this.doc);
/* 130 */       this.props.write(gen);
/* 131 */       gen.writeFieldName("request");
/* 132 */       this.request.fieldsToJson(Protocol.this.types, gen);
/*     */       
/* 134 */       toJson1(gen);
/* 135 */       gen.writeEndObject();
/*     */     }
/*     */     
/*     */     void toJson1(JsonGenerator gen) throws IOException {
/* 139 */       gen.writeStringField("response", "null");
/* 140 */       gen.writeBooleanField("one-way", true);
/*     */     }
/*     */     
/*     */     public boolean equals(Object o) {
/* 144 */       if (o == this) return true;
/* 145 */       if (!(o instanceof Message)) return false;
/* 146 */       Message that = (Message)o;
/* 147 */       return (this.name.equals(that.name)) && (this.request.equals(that.request)) && (this.props.equals(that.props));
/*     */     }
/*     */     
/*     */ 
/*     */     public int hashCode()
/*     */     {
/* 153 */       return this.name.hashCode() + this.request.hashCode() + this.props.hashCode();
/*     */     }
/*     */     
/* 156 */     public String getDoc() { return this.doc; }
/*     */   }
/*     */   
/*     */   private class TwoWayMessage
/*     */     extends Protocol.Message
/*     */   {
/*     */     private Schema response;
/*     */     private Schema errors;
/*     */     
/*     */     private TwoWayMessage(String name, Map<String, String> doc, Schema propMap, Schema request, Schema response)
/*     */     {
/* 167 */       super(name, doc, propMap, request, null);
/* 168 */       this.response = response;
/* 169 */       this.errors = errors;
/*     */     }
/*     */     
/* 172 */     public Schema getResponse() { return this.response; }
/* 173 */     public Schema getErrors() { return this.errors; }
/* 174 */     public boolean isOneWay() { return false; }
/*     */     
/*     */     public boolean equals(Object o) {
/* 177 */       if (!super.equals(o)) return false;
/* 178 */       if (!(o instanceof TwoWayMessage)) return false;
/* 179 */       TwoWayMessage that = (TwoWayMessage)o;
/* 180 */       return (this.response.equals(that.response)) && (this.errors.equals(that.errors));
/*     */     }
/*     */     
/*     */     public int hashCode()
/*     */     {
/* 185 */       return super.hashCode() + this.response.hashCode() + this.errors.hashCode();
/*     */     }
/*     */     
/*     */     void toJson1(JsonGenerator gen) throws IOException {
/* 189 */       gen.writeFieldName("response");
/* 190 */       this.response.toJson(Protocol.this.types, gen);
/*     */       
/* 192 */       List<Schema> errs = this.errors.getTypes();
/* 193 */       if (errs.size() > 1) {
/* 194 */         Schema union = Schema.createUnion(errs.subList(1, errs.size()));
/* 195 */         gen.writeFieldName("errors");
/* 196 */         union.toJson(Protocol.this.types, gen);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 206 */   private Schema.Names types = new Schema.Names();
/* 207 */   private Map<String, Message> messages = new LinkedHashMap();
/*     */   private byte[] md5;
/*     */   public static final Schema SYSTEM_ERROR;
/*     */   public static final Schema SYSTEM_ERRORS;
/*     */   private static final Set<String> PROTOCOL_RESERVED;
/*     */   
/*     */   static
/*     */   {
/*  68 */     Collections.addAll(MESSAGE_RESERVED, new String[] { "doc", "response", "request", "errors", "one-way" });
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 211 */     SYSTEM_ERROR = Schema.create(Schema.Type.STRING);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 216 */     List<Schema> errors = new ArrayList();
/* 217 */     errors.add(SYSTEM_ERROR);
/* 218 */     SYSTEM_ERRORS = Schema.createUnion(errors);
/*     */     
/*     */ 
/* 221 */     PROTOCOL_RESERVED = new HashSet();
/*     */     
/* 223 */     Collections.addAll(PROTOCOL_RESERVED, new String[] { "namespace", "protocol", "doc", "messages", "types", "errors" });
/*     */   }
/*     */   
/*     */ 
/* 227 */   Schema.Props props = new Schema.Props(PROTOCOL_RESERVED);
/*     */   
/*     */ 
/*     */   public Protocol(String name, String doc, String namespace)
/*     */   {
/* 232 */     this.name = name;
/* 233 */     this.doc = doc;
/* 234 */     this.namespace = namespace;
/*     */   }
/*     */   
/* 237 */   public Protocol(String name, String namespace) { this(name, null, namespace); }
/*     */   
/*     */   public String getName()
/*     */   {
/* 241 */     return this.name;
/*     */   }
/*     */   
/* 244 */   public String getNamespace() { return this.namespace; }
/*     */   
/*     */   public String getDoc() {
/* 247 */     return this.doc;
/*     */   }
/*     */   
/* 250 */   public Collection<Schema> getTypes() { return this.types.values(); }
/*     */   
/*     */   public Schema getType(String name) {
/* 253 */     return this.types.get(name);
/*     */   }
/*     */   
/*     */   public void setTypes(Collection<Schema> newTypes) {
/* 257 */     this.types = new Schema.Names();
/* 258 */     for (Schema s : newTypes)
/* 259 */       this.types.add(s);
/*     */   }
/*     */   
/*     */   public Map<String, Message> getMessages() {
/* 263 */     return this.messages;
/*     */   }
/*     */   
/*     */   @Deprecated
/*     */   public Message createMessage(String name, String doc, Schema request) {
/* 268 */     return createMessage(name, doc, new LinkedHashMap(), request);
/*     */   }
/*     */   
/*     */   public Message createMessage(String name, String doc, Map<String, String> propMap, Schema request)
/*     */   {
/* 273 */     return new Message(name, doc, propMap, request, null);
/*     */   }
/*     */   
/*     */ 
/*     */   @Deprecated
/*     */   public Message createMessage(String name, String doc, Schema request, Schema response, Schema errors)
/*     */   {
/* 280 */     return createMessage(name, doc, new LinkedHashMap(), request, response, errors);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Message createMessage(String name, String doc, Map<String, String> propMap, Schema request, Schema response, Schema errors)
/*     */   {
/* 287 */     return new TwoWayMessage(name, doc, propMap, request, response, errors, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public synchronized String getProp(String name)
/*     */   {
/* 296 */     return (String)this.props.get(name);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public synchronized void addProp(String name, String value)
/*     */   {
/* 309 */     this.props.add(name, value);
/*     */   }
/*     */   
/*     */   public Map<String, String> getProps()
/*     */   {
/* 314 */     return Collections.unmodifiableMap(this.props);
/*     */   }
/*     */   
/*     */   public boolean equals(Object o) {
/* 318 */     if (o == this) return true;
/* 319 */     if (!(o instanceof Protocol)) return false;
/* 320 */     Protocol that = (Protocol)o;
/* 321 */     return (this.name.equals(that.name)) && (this.namespace.equals(that.namespace)) && (this.types.equals(that.types)) && (this.messages.equals(that.messages)) && (this.props.equals(that.props));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public int hashCode()
/*     */   {
/* 329 */     return this.name.hashCode() + this.namespace.hashCode() + this.types.hashCode() + this.messages.hashCode() + this.props.hashCode();
/*     */   }
/*     */   
/*     */ 
/*     */   public String toString()
/*     */   {
/* 335 */     return toString(false);
/*     */   }
/*     */   
/*     */   public String toString(boolean pretty)
/*     */   {
/*     */     try
/*     */     {
/* 342 */       StringWriter writer = new StringWriter();
/* 343 */       JsonGenerator gen = Schema.FACTORY.createJsonGenerator(writer);
/* 344 */       if (pretty) gen.useDefaultPrettyPrinter();
/* 345 */       toJson(gen);
/* 346 */       gen.flush();
/* 347 */       return writer.toString();
/*     */     } catch (IOException e) {
/* 349 */       throw new AvroRuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/* 353 */   void toJson(JsonGenerator gen) throws IOException { this.types.space(this.namespace);
/*     */     
/* 355 */     gen.writeStartObject();
/* 356 */     gen.writeStringField("protocol", this.name);
/* 357 */     gen.writeStringField("namespace", this.namespace);
/*     */     
/* 359 */     if (this.doc != null) gen.writeStringField("doc", this.doc);
/* 360 */     this.props.write(gen);
/* 361 */     gen.writeArrayFieldStart("types");
/* 362 */     Schema.Names resolved = new Schema.Names(this.namespace);
/* 363 */     for (Schema type : this.types.values())
/* 364 */       if (!resolved.contains(type))
/* 365 */         type.toJson(resolved, gen);
/* 366 */     gen.writeEndArray();
/*     */     
/* 368 */     gen.writeObjectFieldStart("messages");
/* 369 */     for (Map.Entry<String, Message> e : this.messages.entrySet()) {
/* 370 */       gen.writeFieldName((String)e.getKey());
/* 371 */       ((Message)e.getValue()).toJson(gen);
/*     */     }
/* 373 */     gen.writeEndObject();
/* 374 */     gen.writeEndObject();
/*     */   }
/*     */   
/*     */   public byte[] getMD5()
/*     */   {
/* 379 */     if (this.md5 == null)
/*     */       try {
/* 381 */         this.md5 = MessageDigest.getInstance("MD5").digest(toString().getBytes("UTF-8"));
/*     */       }
/*     */       catch (Exception e) {
/* 384 */         throw new AvroRuntimeException(e);
/*     */       }
/* 386 */     return this.md5;
/*     */   }
/*     */   
/*     */   public static Protocol parse(File file) throws IOException
/*     */   {
/* 391 */     return parse(Schema.FACTORY.createJsonParser(file));
/*     */   }
/*     */   
/*     */   public static Protocol parse(InputStream stream) throws IOException
/*     */   {
/* 396 */     return parse(Schema.FACTORY.createJsonParser(stream));
/*     */   }
/*     */   
/*     */   public static Protocol parse(String string)
/*     */   {
/*     */     try {
/* 402 */       return parse(Schema.FACTORY.createJsonParser(new ByteArrayInputStream(string.getBytes("UTF-8"))));
/*     */     }
/*     */     catch (IOException e) {
/* 405 */       throw new AvroRuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   private static Protocol parse(JsonParser parser) {
/*     */     try {
/* 411 */       Protocol protocol = new Protocol();
/* 412 */       protocol.parse(Schema.MAPPER.readTree(parser));
/* 413 */       return protocol;
/*     */     } catch (IOException e) {
/* 415 */       throw new SchemaParseException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   private void parse(JsonNode json) {
/* 420 */     parseNamespace(json);
/* 421 */     parseName(json);
/* 422 */     parseTypes(json);
/* 423 */     parseMessages(json);
/* 424 */     parseDoc(json);
/* 425 */     parseProps(json);
/*     */   }
/*     */   
/*     */   private void parseNamespace(JsonNode json) {
/* 429 */     JsonNode nameNode = json.get("namespace");
/* 430 */     if (nameNode == null) return;
/* 431 */     this.namespace = nameNode.getTextValue();
/* 432 */     this.types.space(this.namespace);
/*     */   }
/*     */   
/*     */   private void parseDoc(JsonNode json) {
/* 436 */     this.doc = parseDocNode(json);
/*     */   }
/*     */   
/*     */   private String parseDocNode(JsonNode json) {
/* 440 */     JsonNode nameNode = json.get("doc");
/* 441 */     if (nameNode == null) return null;
/* 442 */     return nameNode.getTextValue();
/*     */   }
/*     */   
/*     */   private void parseName(JsonNode json) {
/* 446 */     JsonNode nameNode = json.get("protocol");
/* 447 */     if (nameNode == null)
/* 448 */       throw new SchemaParseException("No protocol name specified: " + json);
/* 449 */     this.name = nameNode.getTextValue();
/*     */   }
/*     */   
/*     */   private void parseTypes(JsonNode json) {
/* 453 */     JsonNode defs = json.get("types");
/* 454 */     if (defs == null) return;
/* 455 */     if (!defs.isArray())
/* 456 */       throw new SchemaParseException("Types not an array: " + defs);
/* 457 */     for (JsonNode type : defs) {
/* 458 */       if (!type.isObject())
/* 459 */         throw new SchemaParseException("Type not an object: " + type);
/* 460 */       Schema.parse(type, this.types);
/*     */     }
/*     */   }
/*     */   
/*     */   private void parseProps(JsonNode json) {
/* 465 */     for (Iterator<String> i = json.getFieldNames(); i.hasNext();) {
/* 466 */       String p = (String)i.next();
/* 467 */       if (!PROTOCOL_RESERVED.contains(p)) {
/* 468 */         JsonNode prop = json.get(p);
/* 469 */         if ((prop.isValueNode()) && (prop.isTextual()))
/* 470 */           addProp(p, prop.getTextValue());
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private void parseMessages(JsonNode json) {
/* 476 */     JsonNode defs = json.get("messages");
/* 477 */     if (defs == null) return;
/* 478 */     for (Iterator<String> i = defs.getFieldNames(); i.hasNext();) {
/* 479 */       String prop = (String)i.next();
/* 480 */       this.messages.put(prop, parseMessage(prop, defs.get(prop)));
/*     */     }
/*     */   }
/*     */   
/*     */   private Message parseMessage(String messageName, JsonNode json) {
/* 485 */     String doc = parseDocNode(json);
/*     */     
/* 487 */     Map<String, String> mProps = new LinkedHashMap();
/* 488 */     for (Iterator<String> i = json.getFieldNames(); i.hasNext();) {
/* 489 */       String p = (String)i.next();
/* 490 */       if (!MESSAGE_RESERVED.contains(p)) {
/* 491 */         JsonNode prop = json.get(p);
/* 492 */         if ((prop.isValueNode()) && (prop.isTextual())) {
/* 493 */           mProps.put(p, prop.getTextValue());
/*     */         }
/*     */       }
/*     */     }
/* 497 */     JsonNode requestNode = json.get("request");
/* 498 */     if ((requestNode == null) || (!requestNode.isArray()))
/* 499 */       throw new SchemaParseException("No request specified: " + json);
/* 500 */     List<Schema.Field> fields = new ArrayList();
/* 501 */     for (JsonNode field : requestNode) {
/* 502 */       JsonNode fieldNameNode = field.get("name");
/* 503 */       if (fieldNameNode == null)
/* 504 */         throw new SchemaParseException("No param name: " + field);
/* 505 */       JsonNode fieldTypeNode = field.get("type");
/* 506 */       if (fieldTypeNode == null)
/* 507 */         throw new SchemaParseException("No param type: " + field);
/* 508 */       String name = fieldNameNode.getTextValue();
/* 509 */       fields.add(new Schema.Field(name, Schema.parse(fieldTypeNode, this.types), null, field.get("default")));
/*     */     }
/*     */     
/*     */ 
/* 513 */     Schema request = Schema.createRecord(fields);
/*     */     
/* 515 */     boolean oneWay = false;
/* 516 */     JsonNode oneWayNode = json.get("one-way");
/* 517 */     if (oneWayNode != null) {
/* 518 */       if (!oneWayNode.isBoolean())
/* 519 */         throw new SchemaParseException("one-way must be boolean: " + json);
/* 520 */       oneWay = oneWayNode.getBooleanValue();
/*     */     }
/*     */     
/* 523 */     JsonNode responseNode = json.get("response");
/* 524 */     if ((!oneWay) && (responseNode == null)) {
/* 525 */       throw new SchemaParseException("No response specified: " + json);
/*     */     }
/* 527 */     JsonNode decls = json.get("errors");
/*     */     
/* 529 */     if (oneWay) {
/* 530 */       if (decls != null)
/* 531 */         throw new SchemaParseException("one-way can't have errors: " + json);
/* 532 */       if ((responseNode != null) && (Schema.parse(responseNode, this.types).getType() != Schema.Type.NULL))
/*     */       {
/* 534 */         throw new SchemaParseException("One way response must be null: " + json); }
/* 535 */       return new Message(messageName, doc, mProps, request, null);
/*     */     }
/*     */     
/* 538 */     Schema response = Schema.parse(responseNode, this.types);
/*     */     
/* 540 */     List<Schema> errs = new ArrayList();
/* 541 */     errs.add(SYSTEM_ERROR);
/* 542 */     if (decls != null) {
/* 543 */       if (!decls.isArray())
/* 544 */         throw new SchemaParseException("Errors not an array: " + json);
/* 545 */       for (JsonNode decl : decls) {
/* 546 */         String name = decl.getTextValue();
/* 547 */         Schema schema = this.types.get(name);
/* 548 */         if (schema == null)
/* 549 */           throw new SchemaParseException("Undefined error: " + name);
/* 550 */         if (!schema.isError())
/* 551 */           throw new SchemaParseException("Not an error: " + name);
/* 552 */         errs.add(schema);
/*     */       }
/*     */     }
/*     */     
/* 556 */     return new TwoWayMessage(messageName, doc, mProps, request, response, Schema.createUnion(errs), null);
/*     */   }
/*     */   
/*     */   public static void main(String[] args) throws Exception
/*     */   {
/* 561 */     System.out.println(parse(new File(args[0])));
/*     */   }
/*     */   
/*     */   private Protocol() {}
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\Protocol.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */