/*     */ package org.apache.avro.io;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.OutputStream;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.nio.channels.Channels;
/*     */ import java.nio.channels.WritableByteChannel;
/*     */ import org.apache.avro.AvroRuntimeException;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BufferedBinaryEncoder
/*     */   extends BinaryEncoder
/*     */ {
/*     */   private byte[] buf;
/*     */   private int pos;
/*     */   private ByteSink sink;
/*     */   private int bulkLimit;
/*     */   
/*     */   BufferedBinaryEncoder(OutputStream out, int bufferSize)
/*     */   {
/*  55 */     configure(out, bufferSize);
/*     */   }
/*     */   
/*     */   BufferedBinaryEncoder configure(OutputStream out, int bufferSize) {
/*  59 */     if (null == out)
/*  60 */       throw new NullPointerException("OutputStream cannot be null!");
/*  61 */     if ((null != this.sink) && 
/*  62 */       (this.pos > 0)) {
/*     */       try {
/*  64 */         flushBuffer();
/*     */       } catch (IOException e) {
/*  66 */         throw new AvroRuntimeException("Failure flushing old output", e);
/*     */       }
/*     */     }
/*     */     
/*  70 */     this.sink = new OutputStreamSink(out, null);
/*  71 */     this.pos = 0;
/*  72 */     if ((null == this.buf) || (this.buf.length != bufferSize)) {
/*  73 */       this.buf = new byte[bufferSize];
/*     */     }
/*  75 */     this.bulkLimit = (this.buf.length >>> 1);
/*  76 */     if (this.bulkLimit > 512) {
/*  77 */       this.bulkLimit = 512;
/*     */     }
/*  79 */     return this;
/*     */   }
/*     */   
/*     */   public void flush() throws IOException
/*     */   {
/*  84 */     flushBuffer();
/*  85 */     this.sink.innerFlush();
/*     */   }
/*     */   
/*     */ 
/*     */   private void flushBuffer()
/*     */     throws IOException
/*     */   {
/*  92 */     if (this.pos > 0) {
/*  93 */       this.sink.innerWrite(this.buf, 0, this.pos);
/*  94 */       this.pos = 0;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void ensureBounds(int num)
/*     */     throws IOException
/*     */   {
/* 106 */     int remaining = this.buf.length - this.pos;
/* 107 */     if (remaining < num) {
/* 108 */       flushBuffer();
/*     */     }
/*     */   }
/*     */   
/*     */   public void writeBoolean(boolean b)
/*     */     throws IOException
/*     */   {
/* 115 */     if (this.buf.length == this.pos) {
/* 116 */       flushBuffer();
/*     */     }
/* 118 */     this.pos += BinaryData.encodeBoolean(b, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeInt(int n) throws IOException
/*     */   {
/* 123 */     ensureBounds(5);
/* 124 */     this.pos += BinaryData.encodeInt(n, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeLong(long n) throws IOException
/*     */   {
/* 129 */     ensureBounds(10);
/* 130 */     this.pos += BinaryData.encodeLong(n, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeFloat(float f) throws IOException
/*     */   {
/* 135 */     ensureBounds(4);
/* 136 */     this.pos += BinaryData.encodeFloat(f, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeDouble(double d) throws IOException
/*     */   {
/* 141 */     ensureBounds(8);
/* 142 */     this.pos += BinaryData.encodeDouble(d, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeFixed(byte[] bytes, int start, int len) throws IOException
/*     */   {
/* 147 */     if (len > this.bulkLimit)
/*     */     {
/* 149 */       flushBuffer();
/* 150 */       this.sink.innerWrite(bytes, start, len);
/* 151 */       return;
/*     */     }
/* 153 */     ensureBounds(len);
/* 154 */     System.arraycopy(bytes, start, this.buf, this.pos, len);
/* 155 */     this.pos += len;
/*     */   }
/*     */   
/*     */   public void writeFixed(ByteBuffer bytes) throws IOException
/*     */   {
/* 160 */     if ((!bytes.hasArray()) && (bytes.remaining() > this.bulkLimit)) {
/* 161 */       this.sink.innerWrite(bytes);
/*     */     } else {
/* 163 */       super.writeFixed(bytes);
/*     */     }
/*     */   }
/*     */   
/*     */   protected void writeZero() throws IOException
/*     */   {
/* 169 */     writeByte(0);
/*     */   }
/*     */   
/*     */   private void writeByte(int b) throws IOException {
/* 173 */     if (this.pos == this.buf.length) {
/* 174 */       flushBuffer();
/*     */     }
/* 176 */     this.buf[(this.pos++)] = ((byte)(b & 0xFF));
/*     */   }
/*     */   
/*     */   public int bytesBuffered()
/*     */   {
/* 181 */     return this.pos;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static class OutputStreamSink
/*     */     extends BufferedBinaryEncoder.ByteSink
/*     */   {
/*     */     private final OutputStream out;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     private final WritableByteChannel channel;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     private OutputStreamSink(OutputStream out)
/*     */     {
/* 209 */       this.out = out;
/* 210 */       this.channel = Channels.newChannel(out);
/*     */     }
/*     */     
/*     */     protected void innerWrite(byte[] bytes, int off, int len) throws IOException
/*     */     {
/* 215 */       this.out.write(bytes, off, len);
/*     */     }
/*     */     
/*     */     protected void innerFlush() throws IOException {
/* 219 */       this.out.flush();
/*     */     }
/*     */     
/*     */     protected void innerWrite(ByteBuffer buff) throws IOException {
/* 223 */       this.channel.write(buff);
/*     */     }
/*     */   }
/*     */   
/*     */   private static abstract class ByteSink
/*     */   {
/*     */     protected abstract void innerWrite(byte[] paramArrayOfByte, int paramInt1, int paramInt2)
/*     */       throws IOException;
/*     */     
/*     */     protected abstract void innerWrite(ByteBuffer paramByteBuffer)
/*     */       throws IOException;
/*     */     
/*     */     protected abstract void innerFlush()
/*     */       throws IOException;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\BufferedBinaryEncoder.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */