package javax.wsdl.extensions;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;
import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;

public class ExtensionRegistry
  implements Serializable
{
  public static final long serialVersionUID = 1L;
  protected Map serializerReg = new Hashtable();
  protected Map deserializerReg = new Hashtable();
  protected Map extensionTypeReg = new Hashtable();
  protected ExtensionSerializer defaultSer = null;
  protected ExtensionDeserializer defaultDeser = null;
  protected Map extensionAttributeTypeReg = new Hashtable();
  
  public ExtensionRegistry()
  {
    setDefaultSerializer(new UnknownExtensionSerializer());
    setDefaultDeserializer(new UnknownExtensionDeserializer());
  }
  
  public void setDefaultSerializer(ExtensionSerializer paramExtensionSerializer)
  {
    this.defaultSer = paramExtensionSerializer;
  }
  
  public ExtensionSerializer getDefaultSerializer()
  {
    return this.defaultSer;
  }
  
  public void setDefaultDeserializer(ExtensionDeserializer paramExtensionDeserializer)
  {
    this.defaultDeser = paramExtensionDeserializer;
  }
  
  public ExtensionDeserializer getDefaultDeserializer()
  {
    return this.defaultDeser;
  }
  
  public void registerSerializer(Class paramClass, QName paramQName, ExtensionSerializer paramExtensionSerializer)
  {
    Object localObject = (Map)this.serializerReg.get(paramClass);
    if (localObject == null)
    {
      localObject = new Hashtable();
      this.serializerReg.put(paramClass, localObject);
    }
    ((Map)localObject).put(paramQName, paramExtensionSerializer);
  }
  
  public void registerDeserializer(Class paramClass, QName paramQName, ExtensionDeserializer paramExtensionDeserializer)
  {
    Object localObject = (Map)this.deserializerReg.get(paramClass);
    if (localObject == null)
    {
      localObject = new Hashtable();
      this.deserializerReg.put(paramClass, localObject);
    }
    ((Map)localObject).put(paramQName, paramExtensionDeserializer);
  }
  
  public ExtensionSerializer querySerializer(Class paramClass, QName paramQName)
    throws WSDLException
  {
    Map localMap = (Map)this.serializerReg.get(paramClass);
    ExtensionSerializer localExtensionSerializer = null;
    if (localMap != null) {
      localExtensionSerializer = (ExtensionSerializer)localMap.get(paramQName);
    }
    if (localExtensionSerializer == null) {
      localExtensionSerializer = this.defaultSer;
    }
    if (localExtensionSerializer == null) {
      throw new WSDLException("CONFIGURATION_ERROR", "No ExtensionSerializer found to serialize a '" + paramQName + "' element in the context of a '" + paramClass.getName() + "'.");
    }
    return localExtensionSerializer;
  }
  
  public ExtensionDeserializer queryDeserializer(Class paramClass, QName paramQName)
    throws WSDLException
  {
    Map localMap = (Map)this.deserializerReg.get(paramClass);
    ExtensionDeserializer localExtensionDeserializer = null;
    if (localMap != null) {
      localExtensionDeserializer = (ExtensionDeserializer)localMap.get(paramQName);
    }
    if (localExtensionDeserializer == null) {
      localExtensionDeserializer = this.defaultDeser;
    }
    if (localExtensionDeserializer == null) {
      throw new WSDLException("CONFIGURATION_ERROR", "No ExtensionDeserializer found to deserialize a '" + paramQName + "' element in the context of a '" + paramClass.getName() + "'.");
    }
    return localExtensionDeserializer;
  }
  
  public Set getAllowableExtensions(Class paramClass)
  {
    Map localMap = (Map)this.deserializerReg.get(paramClass);
    return localMap != null ? localMap.keySet() : null;
  }
  
  public void mapExtensionTypes(Class paramClass1, QName paramQName, Class paramClass2)
  {
    Object localObject = (Map)this.extensionTypeReg.get(paramClass1);
    if (localObject == null)
    {
      localObject = new Hashtable();
      this.extensionTypeReg.put(paramClass1, localObject);
    }
    ((Map)localObject).put(paramQName, paramClass2);
  }
  
  public ExtensibilityElement createExtension(Class paramClass, QName paramQName)
    throws WSDLException
  {
    Map localMap = (Map)this.extensionTypeReg.get(paramClass);
    Class localClass = null;
    if (localMap != null) {
      localClass = (Class)localMap.get(paramQName);
    }
    if (localClass == null) {
      throw new WSDLException("CONFIGURATION_ERROR", "No Java extensionType found to represent a '" + paramQName + "' element in the context of a '" + paramClass.getName() + "'.");
    }
    if (!ExtensibilityElement.class.isAssignableFrom(localClass)) {
      throw new WSDLException("CONFIGURATION_ERROR", "The Java extensionType '" + localClass.getName() + "' does " + "not implement the ExtensibilityElement " + "interface.");
    }
    try
    {
      ExtensibilityElement localExtensibilityElement = (ExtensibilityElement)localClass.newInstance();
      if (localExtensibilityElement.getElementType() == null) {
        localExtensibilityElement.setElementType(paramQName);
      }
      return localExtensibilityElement;
    }
    catch (Exception localException)
    {
      throw new WSDLException("CONFIGURATION_ERROR", "Problem instantiating Java extensionType '" + localClass.getName() + "'.", localException);
    }
  }
  
  public void registerExtensionAttributeType(Class paramClass, QName paramQName, int paramInt)
  {
    Object localObject = (Map)this.extensionAttributeTypeReg.get(paramClass);
    if (localObject == null)
    {
      localObject = new Hashtable();
      this.extensionAttributeTypeReg.put(paramClass, localObject);
    }
    ((Map)localObject).put(paramQName, new Integer(paramInt));
  }
  
  public int queryExtensionAttributeType(Class paramClass, QName paramQName)
  {
    Map localMap = (Map)this.extensionAttributeTypeReg.get(paramClass);
    Integer localInteger = null;
    if (localMap != null) {
      localInteger = (Integer)localMap.get(paramQName);
    }
    if (localInteger != null) {
      return localInteger.intValue();
    }
    return -1;
  }
}


/* Location:              /Users/anandkum/Downloads/axis-wsdl4j-1.5.1.jar!/javax/wsdl/extensions/ExtensionRegistry.class
 * Java compiler version: 2 (46.0)
 * JD-Core Version:       0.7.1
 */