package prefuse.action.filter;

import java.util.Iterator;
import prefuse.Visualization;
import prefuse.action.GroupAction;
import prefuse.data.Graph;
import prefuse.data.Tree;
import prefuse.data.expression.Predicate;
import prefuse.util.PrefuseLib;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;

public class FisheyeTreeFilter
  extends GroupAction
{
  private String m_sources;
  private Predicate m_groupP;
  private int m_threshold;
  private NodeItem m_root;
  private double m_divisor;
  
  public FisheyeTreeFilter(String paramString)
  {
    this(paramString, 1);
  }
  
  public FisheyeTreeFilter(String paramString, int paramInt)
  {
    this(paramString, Visualization.FOCUS_ITEMS, paramInt);
  }
  
  public FisheyeTreeFilter(String paramString1, String paramString2, int paramInt)
  {
    super(paramString1);
    this.m_sources = paramString2;
    this.m_threshold = (-paramInt);
    this.m_groupP = new InGroupPredicate(PrefuseLib.getGroupName(paramString1, Graph.NODES));
  }
  
  public void run(double paramDouble)
  {
    Tree localTree = ((Graph)this.m_vis.getGroup(this.m_group)).getSpanningTree();
    this.m_divisor = localTree.getNodeCount();
    this.m_root = ((NodeItem)localTree.getRoot());
    Iterator localIterator = this.m_vis.visibleItems(this.m_group);
    while (localIterator.hasNext())
    {
      localObject = (VisualItem)localIterator.next();
      ((VisualItem)localObject).setDOI(-1.7976931348623157E308D);
      ((VisualItem)localObject).setExpanded(false);
    }
    Object localObject = this.m_vis.items(this.m_sources, this.m_groupP);
    while (((Iterator)localObject).hasNext()) {
      visitFocus((NodeItem)((Iterator)localObject).next(), null);
    }
    visitFocus(this.m_root, null);
    localIterator = this.m_vis.visibleItems(this.m_group);
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem = (VisualItem)localIterator.next();
      if (localVisualItem.getDOI() == -1.7976931348623157E308D) {
        PrefuseLib.updateVisible(localVisualItem, false);
      }
    }
  }
  
  private void visitFocus(NodeItem paramNodeItem1, NodeItem paramNodeItem2)
  {
    if (paramNodeItem1.getDOI() <= -1.0D)
    {
      visit(paramNodeItem1, paramNodeItem2, 0, 0);
      if (this.m_threshold < 0) {
        visitDescendants(paramNodeItem1, paramNodeItem2);
      }
      visitAncestors(paramNodeItem1);
    }
  }
  
  private void visit(NodeItem paramNodeItem1, NodeItem paramNodeItem2, int paramInt1, int paramInt2)
  {
    PrefuseLib.updateVisible(paramNodeItem1, true);
    double d = -paramInt2 / Math.min(1000.0D, this.m_divisor);
    paramNodeItem1.setDOI(paramInt1 + d);
    if (paramNodeItem2 != null)
    {
      EdgeItem localEdgeItem = (EdgeItem)paramNodeItem2.getParentEdge();
      localEdgeItem.setDOI(paramNodeItem2.getDOI());
      PrefuseLib.updateVisible(localEdgeItem, true);
    }
  }
  
  private void visitAncestors(NodeItem paramNodeItem)
  {
    if (paramNodeItem == this.m_root) {
      return;
    }
    visitFocus((NodeItem)paramNodeItem.getParent(), paramNodeItem);
  }
  
  private void visitDescendants(NodeItem paramNodeItem1, NodeItem paramNodeItem2)
  {
    int i = paramNodeItem2 == null ? 0 : paramNodeItem1.getChildIndex(paramNodeItem2);
    Iterator localIterator = paramNodeItem1.children();
    paramNodeItem1.setExpanded(localIterator.hasNext());
    for (int j = 0; localIterator.hasNext(); j++)
    {
      NodeItem localNodeItem = (NodeItem)localIterator.next();
      if (localNodeItem != paramNodeItem2)
      {
        int k = (int)(paramNodeItem1.getDOI() - 1.0D);
        visit(localNodeItem, localNodeItem, k, Math.abs(i - j));
        if (k > this.m_threshold) {
          visitDescendants(localNodeItem, null);
        }
      }
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\filter\FisheyeTreeFilter.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */