package prefuse.action.layout.graph;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;
import prefuse.Visualization;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.tuple.TupleSet;
import prefuse.util.ArrayLib;
import prefuse.visual.NodeItem;

public class RadialTreeLayout
  extends TreeLayout
{
  public static final int DEFAULT_RADIUS = 50;
  protected int m_maxDepth = 0;
  protected double m_radiusInc = 50.0D;
  protected double m_theta1 = 0.0D;
  protected double m_theta2 = this.m_theta1 + 6.283185307179586D;
  protected boolean m_setTheta = false;
  protected boolean m_autoScale = true;
  protected Point2D m_origin;
  protected NodeItem m_prevRoot = null;
  public static final String PARAMS = "_radialTreeLayoutParams";
  public static final Schema PARAMS_SCHEMA = new Schema();
  
  public RadialTreeLayout(String paramString)
  {
    super(paramString);
  }
  
  public RadialTreeLayout(String paramString, int paramInt)
  {
    this(paramString);
    this.m_radiusInc = paramInt;
    this.m_autoScale = false;
  }
  
  public double getRadiusIncrement()
  {
    return this.m_radiusInc;
  }
  
  public void setRadiusIncrement(double paramDouble)
  {
    this.m_radiusInc = paramDouble;
  }
  
  public boolean getAutoScale()
  {
    return this.m_autoScale;
  }
  
  public void setAutoScale(boolean paramBoolean)
  {
    this.m_autoScale = paramBoolean;
  }
  
  public void setAngularBounds(double paramDouble1, double paramDouble2)
  {
    this.m_theta1 = paramDouble1;
    this.m_theta2 = (paramDouble1 + paramDouble2);
    this.m_setTheta = true;
  }
  
  public void run(double paramDouble)
  {
    Graph localGraph = (Graph)this.m_vis.getGroup(this.m_group);
    initSchema(localGraph.getNodes());
    this.m_origin = getLayoutAnchor();
    NodeItem localNodeItem = getLayoutRoot();
    Params localParams = (Params)localNodeItem.get("_radialTreeLayoutParams");
    this.m_maxDepth = 0;
    calcAngularWidth(localNodeItem, 0);
    if (this.m_autoScale) {
      setScale(getLayoutBounds());
    }
    if (!this.m_setTheta) {
      calcAngularBounds(localNodeItem);
    }
    if (this.m_maxDepth > 0) {
      layout(localNodeItem, this.m_radiusInc, this.m_theta1, this.m_theta2);
    }
    setX(localNodeItem, null, this.m_origin.getX());
    setY(localNodeItem, null, this.m_origin.getY());
    localParams.angle = (this.m_theta2 - this.m_theta1);
  }
  
  protected void setScale(Rectangle2D paramRectangle2D)
  {
    double d = Math.min(paramRectangle2D.getWidth(), paramRectangle2D.getHeight()) / 2.0D;
    if (this.m_maxDepth > 0) {
      this.m_radiusInc = ((d - 30.0D) / this.m_maxDepth);
    }
  }
  
  private void calcAngularBounds(NodeItem paramNodeItem)
  {
    if ((this.m_prevRoot == null) || (paramNodeItem == this.m_prevRoot))
    {
      this.m_prevRoot = paramNodeItem;
      return;
    }
    NodeItem localNodeItem;
    for (Object localObject = this.m_prevRoot;; localObject = localNodeItem)
    {
      localNodeItem = (NodeItem)((NodeItem)localObject).getParent();
      if (localNodeItem == paramNodeItem) {
        break;
      }
      if (localNodeItem == null)
      {
        this.m_prevRoot = paramNodeItem;
        return;
      }
    }
    double d1 = 0.0D;
    Iterator localIterator = sortedChildren(paramNodeItem);
    while (localIterator.hasNext())
    {
      Node localNode = (Node)localIterator.next();
      if (localNode == localObject) {
        break;
      }
      d1 += ((Params)localNode.get("_radialTreeLayoutParams")).width;
    }
    double d2 = ((Params)paramNodeItem.get("_radialTreeLayoutParams")).width;
    double d3 = ((Params)((NodeItem)localObject).get("_radialTreeLayoutParams")).width;
    d1 = -6.283185307179586D * (d1 + d3 / 2.0D) / d2;
    this.m_theta1 = (d1 + Math.atan2(((NodeItem)localObject).getY() - paramNodeItem.getY(), ((NodeItem)localObject).getX() - paramNodeItem.getX()));
    this.m_theta2 = (this.m_theta1 + 6.283185307179586D);
    this.m_prevRoot = paramNodeItem;
  }
  
  private double calcAngularWidth(NodeItem paramNodeItem, int paramInt)
  {
    if (paramInt > this.m_maxDepth) {
      this.m_maxDepth = paramInt;
    }
    double d1 = 0.0D;
    Rectangle2D localRectangle2D = paramNodeItem.getBounds();
    double d2 = localRectangle2D.getWidth();
    double d3 = localRectangle2D.getHeight();
    double d4 = paramInt == 0 ? 0.0D : Math.sqrt(d2 * d2 + d3 * d3) / paramInt;
    if ((paramNodeItem.isExpanded()) && (paramNodeItem.getChildCount() > 0))
    {
      Iterator localIterator = paramNodeItem.children();
      while (localIterator.hasNext())
      {
        NodeItem localNodeItem = (NodeItem)localIterator.next();
        d1 += calcAngularWidth(localNodeItem, paramInt + 1);
      }
      d1 = Math.max(d4, d1);
    }
    else
    {
      d1 = d4;
    }
    ((Params)paramNodeItem.get("_radialTreeLayoutParams")).width = d1;
    return d1;
  }
  
  private static final double normalize(double paramDouble)
  {
    while (paramDouble > 6.283185307179586D) {
      paramDouble -= 6.283185307179586D;
    }
    while (paramDouble < 0.0D) {
      paramDouble += 6.283185307179586D;
    }
    return paramDouble;
  }
  
  private Iterator sortedChildren(final NodeItem paramNodeItem)
  {
    double d = 0.0D;
    NodeItem localNodeItem1 = (NodeItem)paramNodeItem.getParent();
    if (localNodeItem1 != null) {
      d = normalize(Math.atan2(localNodeItem1.getY() - paramNodeItem.getY(), localNodeItem1.getX() - paramNodeItem.getX()));
    }
    int i = paramNodeItem.getChildCount();
    if (i == 0) {
      return null;
    }
    NodeItem localNodeItem2 = (NodeItem)paramNodeItem.getFirstChild();
    if (!localNodeItem2.isStartVisible()) {
      return paramNodeItem.children();
    }
    double[] arrayOfDouble = new double[i];
    final int[] arrayOfInt = new int[i];
    int j = 0;
    while (j < i)
    {
      arrayOfInt[j] = j;
      arrayOfDouble[j] = normalize(-d + Math.atan2(localNodeItem2.getY() - paramNodeItem.getY(), localNodeItem2.getX() - paramNodeItem.getX()));
      j++;
      localNodeItem2 = (NodeItem)localNodeItem2.getNextSibling();
    }
    ArrayLib.sort(arrayOfDouble, arrayOfInt);
    new Iterator()
    {
      int cur = 0;
      
      public Object next()
      {
        return paramNodeItem.getChild(arrayOfInt[(this.cur++)]);
      }
      
      public boolean hasNext()
      {
        return this.cur < arrayOfInt.length;
      }
      
      public void remove()
      {
        throw new UnsupportedOperationException();
      }
    };
  }
  
  protected void layout(NodeItem paramNodeItem, double paramDouble1, double paramDouble2, double paramDouble3)
  {
    double d1 = paramDouble3 - paramDouble2;
    double d2 = d1 / 2.0D;
    double d3 = ((Params)paramNodeItem.get("_radialTreeLayoutParams")).width;
    double d5 = 0.0D;
    Iterator localIterator = sortedChildren(paramNodeItem);
    while ((localIterator != null) && (localIterator.hasNext()))
    {
      NodeItem localNodeItem = (NodeItem)localIterator.next();
      Params localParams = (Params)localNodeItem.get("_radialTreeLayoutParams");
      double d4 = localParams.width / d3;
      if ((localNodeItem.isExpanded()) && (localNodeItem.getChildCount() > 0)) {
        layout(localNodeItem, paramDouble1 + this.m_radiusInc, paramDouble2 + d5 * d1, paramDouble2 + (d5 + d4) * d1);
      }
      setPolarLocation(localNodeItem, paramNodeItem, paramDouble1, paramDouble2 + d5 * d1 + d4 * d2);
      localParams.angle = (d4 * d1);
      d5 += d4;
    }
  }
  
  protected void setPolarLocation(NodeItem paramNodeItem1, NodeItem paramNodeItem2, double paramDouble1, double paramDouble2)
  {
    setX(paramNodeItem1, paramNodeItem2, this.m_origin.getX() + paramDouble1 * Math.cos(paramDouble2));
    setY(paramNodeItem1, paramNodeItem2, this.m_origin.getY() + paramDouble1 * Math.sin(paramDouble2));
  }
  
  protected void initSchema(TupleSet paramTupleSet)
  {
    paramTupleSet.addColumns(PARAMS_SCHEMA);
  }
  
  static
  {
    PARAMS_SCHEMA.addColumn("_radialTreeLayoutParams", Params.class, new Params());
  }
  
  public static class Params
    implements Cloneable
  {
    double width;
    double angle;
    
    public Object clone()
    {
      Params localParams = new Params();
      localParams.width = this.width;
      localParams.angle = this.angle;
      return localParams;
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\layout\graph\RadialTreeLayout.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */