package prefuse.data.column;

import java.util.Date;
import prefuse.data.DataTypeException;
import prefuse.data.event.ColumnListener;
import prefuse.util.TypeLib;
import prefuse.util.collections.CopyOnWriteArrayList;

public abstract class AbstractColumn
  implements Column
{
  protected final Class m_columnClass;
  protected Object m_defaultValue;
  protected boolean m_readOnly;
  protected CopyOnWriteArrayList m_listeners;
  
  public AbstractColumn()
  {
    this(Object.class, null);
  }
  
  public AbstractColumn(Class paramClass)
  {
    this(paramClass, null);
  }
  
  public AbstractColumn(Class paramClass, Object paramObject)
  {
    this.m_columnClass = paramClass;
    setDefaultValue(paramObject);
    this.m_readOnly = false;
    this.m_listeners = new CopyOnWriteArrayList();
  }
  
  public boolean isReadOnly()
  {
    return this.m_readOnly;
  }
  
  public void setReadOnly(boolean paramBoolean)
  {
    this.m_readOnly = paramBoolean;
  }
  
  public boolean isCellEditable(int paramInt)
  {
    return !this.m_readOnly;
  }
  
  public Class getColumnClass()
  {
    return this.m_columnClass;
  }
  
  public void addColumnListener(ColumnListener paramColumnListener)
  {
    this.m_listeners.add(paramColumnListener);
  }
  
  public void removeColumnListener(ColumnListener paramColumnListener)
  {
    this.m_listeners.remove(paramColumnListener);
  }
  
  protected final void fireColumnEvent(int paramInt1, int paramInt2, int paramInt3)
  {
    Object[] arrayOfObject = this.m_listeners.getArray();
    for (int i = 0; i < arrayOfObject.length; i++) {
      ((ColumnListener)arrayOfObject[i]).columnChanged(this, paramInt1, paramInt2, paramInt3);
    }
  }
  
  protected final void fireColumnEvent(int paramInt1, int paramInt2)
  {
    Object[] arrayOfObject = this.m_listeners.getArray();
    for (int i = 0; i < arrayOfObject.length; i++) {
      ((ColumnListener)arrayOfObject[i]).columnChanged(this, paramInt1, paramInt2);
    }
  }
  
  protected final void fireColumnEvent(int paramInt, long paramLong)
  {
    Object[] arrayOfObject = this.m_listeners.getArray();
    for (int i = 0; i < arrayOfObject.length; i++) {
      ((ColumnListener)arrayOfObject[i]).columnChanged(this, paramInt, paramLong);
    }
  }
  
  protected final void fireColumnEvent(int paramInt, float paramFloat)
  {
    Object[] arrayOfObject = this.m_listeners.getArray();
    for (int i = 0; i < arrayOfObject.length; i++) {
      ((ColumnListener)arrayOfObject[i]).columnChanged(this, paramInt, paramFloat);
    }
  }
  
  protected final void fireColumnEvent(int paramInt, double paramDouble)
  {
    Object[] arrayOfObject = this.m_listeners.getArray();
    for (int i = 0; i < arrayOfObject.length; i++) {
      ((ColumnListener)arrayOfObject[i]).columnChanged(this, paramInt, paramDouble);
    }
  }
  
  protected final void fireColumnEvent(int paramInt, boolean paramBoolean)
  {
    Object[] arrayOfObject = this.m_listeners.getArray();
    for (int i = 0; i < arrayOfObject.length; i++) {
      ((ColumnListener)arrayOfObject[i]).columnChanged(this, paramInt, paramBoolean);
    }
  }
  
  protected final void fireColumnEvent(int paramInt, Object paramObject)
  {
    Object[] arrayOfObject = this.m_listeners.getArray();
    for (int i = 0; i < arrayOfObject.length; i++) {
      ((ColumnListener)arrayOfObject[i]).columnChanged(this, paramInt, paramObject);
    }
  }
  
  public Object getDefaultValue()
  {
    return this.m_defaultValue;
  }
  
  public void setDefaultValue(Object paramObject)
  {
    boolean bool = this.m_columnClass.isPrimitive();
    if ((paramObject != null) && (((!bool) && (!this.m_columnClass.isInstance(paramObject))) || ((bool) && (!TypeLib.isWrapperInstance(this.m_columnClass, paramObject))))) {
      throw new IllegalArgumentException("Default value is not of type " + this.m_columnClass.getName());
    }
    this.m_defaultValue = paramObject;
  }
  
  public void revertToDefault(int paramInt)
  {
    set(this.m_defaultValue, paramInt);
  }
  
  public boolean canGet(Class paramClass)
  {
    if (paramClass == null) {
      return false;
    }
    if (this.m_columnClass.isPrimitive())
    {
      int i = (paramClass.isAssignableFrom(this.m_columnClass)) || ((TypeLib.isNumericType(this.m_columnClass)) && (TypeLib.isNumericType(paramClass))) ? 1 : 0;
      return (i != 0) || (paramClass.isAssignableFrom(TypeLib.getWrapperType(this.m_columnClass))) || (paramClass.isAssignableFrom(String.class));
    }
    return paramClass.isAssignableFrom(this.m_columnClass);
  }
  
  public boolean canSet(Class paramClass)
  {
    if (paramClass == null) {
      return false;
    }
    if (this.m_columnClass.isPrimitive()) {
      return (this.m_columnClass.isAssignableFrom(paramClass)) || (TypeLib.getWrapperType(this.m_columnClass).isAssignableFrom(paramClass)) || (String.class.isAssignableFrom(paramClass));
    }
    return this.m_columnClass.isAssignableFrom(paramClass);
  }
  
  public boolean canGetInt()
  {
    return canGet(Integer.TYPE);
  }
  
  public boolean canSetInt()
  {
    return canSet(Integer.TYPE);
  }
  
  public int getInt(int paramInt)
    throws DataTypeException
  {
    if (canGetInt()) {
      return ((Integer)get(paramInt)).intValue();
    }
    throw new DataTypeException(Integer.TYPE);
  }
  
  public void setInt(int paramInt1, int paramInt2)
    throws DataTypeException
  {
    if (canSetInt()) {
      set(new Integer(paramInt1), paramInt2);
    } else {
      throw new DataTypeException(Integer.TYPE);
    }
  }
  
  public boolean canGetLong()
  {
    return canGet(Long.TYPE);
  }
  
  public boolean canSetLong()
  {
    return canSet(Long.TYPE);
  }
  
  public long getLong(int paramInt)
    throws DataTypeException
  {
    if (canGetLong()) {
      return ((Long)get(paramInt)).longValue();
    }
    throw new DataTypeException(Long.TYPE);
  }
  
  public void setLong(long paramLong, int paramInt)
    throws DataTypeException
  {
    if (canSetLong()) {
      set(new Long(paramLong), paramInt);
    } else {
      throw new DataTypeException(Long.TYPE);
    }
  }
  
  public boolean canGetFloat()
  {
    return canGet(Float.TYPE);
  }
  
  public boolean canSetFloat()
  {
    return canSet(Float.TYPE);
  }
  
  public float getFloat(int paramInt)
    throws DataTypeException
  {
    if (canGetFloat()) {
      return ((Float)get(paramInt)).floatValue();
    }
    throw new DataTypeException(Float.TYPE);
  }
  
  public void setFloat(float paramFloat, int paramInt)
    throws DataTypeException
  {
    if (canSetFloat()) {
      set(new Float(paramFloat), paramInt);
    } else {
      throw new DataTypeException(Float.TYPE);
    }
  }
  
  public boolean canGetDouble()
  {
    return canGet(Double.TYPE);
  }
  
  public boolean canSetDouble()
  {
    return canSet(Double.TYPE);
  }
  
  public double getDouble(int paramInt)
    throws DataTypeException
  {
    if (canGetDouble()) {
      return ((Double)get(paramInt)).doubleValue();
    }
    throw new DataTypeException(Double.TYPE);
  }
  
  public void setDouble(double paramDouble, int paramInt)
    throws DataTypeException
  {
    if (canSetDouble()) {
      set(new Double(paramDouble), paramInt);
    } else {
      throw new DataTypeException(Double.TYPE);
    }
  }
  
  public boolean canGetBoolean()
  {
    return canGet(Boolean.TYPE);
  }
  
  public boolean canSetBoolean()
  {
    return canSet(Boolean.TYPE);
  }
  
  public boolean getBoolean(int paramInt)
    throws DataTypeException
  {
    if (canGetBoolean()) {
      return ((Boolean)get(paramInt)).booleanValue();
    }
    throw new DataTypeException(Boolean.TYPE);
  }
  
  public void setBoolean(boolean paramBoolean, int paramInt)
    throws DataTypeException
  {
    if (canSetBoolean()) {
      set(new Boolean(paramBoolean), paramInt);
    } else {
      throw new DataTypeException(Boolean.TYPE);
    }
  }
  
  public boolean canGetString()
  {
    return canGet(String.class);
  }
  
  public boolean canSetString()
  {
    return canSet(String.class);
  }
  
  public String getString(int paramInt)
    throws DataTypeException
  {
    if (canGetString())
    {
      Object localObject = get(paramInt);
      return localObject == null ? "" : localObject.toString();
    }
    throw new DataTypeException(String.class);
  }
  
  public void setString(String paramString, int paramInt)
    throws DataTypeException
  {
    set(paramString, paramInt);
  }
  
  public boolean canGetDate()
  {
    return canGet(Date.class);
  }
  
  public boolean canSetDate()
  {
    return canSet(Date.class);
  }
  
  public Date getDate(int paramInt)
    throws DataTypeException
  {
    if (canGetDate()) {
      return (Date)get(paramInt);
    }
    throw new DataTypeException(Date.class);
  }
  
  public void setDate(Date paramDate, int paramInt)
    throws DataTypeException
  {
    set(paramDate, paramInt);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\column\AbstractColumn.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */