package prefuse.data.expression;

import java.util.Comparator;
import prefuse.data.Schema;
import prefuse.data.Tuple;
import prefuse.util.TypeLib;
import prefuse.util.collections.DefaultLiteralComparator;
import prefuse.util.collections.LiteralComparator;

public class RangePredicate
  extends BinaryExpression
  implements Predicate
{
  public static final int IN_IN = 0;
  public static final int IN_EX = 1;
  public static final int EX_IN = 2;
  public static final int EX_EX = 3;
  private Expression m_middle;
  private Comparator m_cmp;
  
  public RangePredicate(Expression paramExpression1, Expression paramExpression2, Expression paramExpression3)
  {
    this(0, paramExpression1, paramExpression2, paramExpression3, DefaultLiteralComparator.getInstance());
  }
  
  public RangePredicate(Expression paramExpression1, Expression paramExpression2, Expression paramExpression3, Comparator paramComparator)
  {
    this(0, paramExpression1, paramExpression2, paramExpression3, paramComparator);
  }
  
  public RangePredicate(int paramInt, Expression paramExpression1, Expression paramExpression2, Expression paramExpression3)
  {
    this(paramInt, paramExpression1, paramExpression2, paramExpression3, DefaultLiteralComparator.getInstance());
  }
  
  public RangePredicate(int paramInt, Expression paramExpression1, Expression paramExpression2, Expression paramExpression3, Comparator paramComparator)
  {
    super(paramInt, 0, 3, paramExpression2, paramExpression3);
    this.m_middle = paramExpression1;
    this.m_cmp = paramComparator;
  }
  
  public Expression getMiddleExpression()
  {
    return this.m_middle;
  }
  
  public Comparator getComparator()
  {
    return this.m_cmp;
  }
  
  public boolean getBoolean(Tuple paramTuple)
  {
    Class localClass1 = this.m_left.getType(paramTuple.getSchema());
    Class localClass2 = this.m_right.getType(paramTuple.getSchema());
    Class localClass3 = this.m_middle.getType(paramTuple.getSchema());
    Class localClass4 = null;
    if (localClass1.isAssignableFrom(localClass2)) {
      localClass4 = localClass1;
    } else if (localClass2.isAssignableFrom(localClass1)) {
      localClass4 = localClass2;
    }
    int j = 0;
    Object localObject1;
    int i;
    if ((localClass4 != null) && (TypeLib.isNumericType(localClass4)) && (TypeLib.isNumericType(localClass3)))
    {
      localObject1 = TypeLib.getNumericType(localClass4, localClass3);
      if (localObject1 == Integer.TYPE)
      {
        int k = this.m_left.getInt(paramTuple);
        int m = this.m_right.getInt(paramTuple);
        int n = this.m_middle.getInt(paramTuple);
        i = ((LiteralComparator)this.m_cmp).compare(n, k);
        j = ((LiteralComparator)this.m_cmp).compare(n, m);
      }
      else if (localObject1 == Long.TYPE)
      {
        long l1 = this.m_left.getLong(paramTuple);
        long l2 = this.m_right.getLong(paramTuple);
        long l3 = this.m_middle.getLong(paramTuple);
        i = ((LiteralComparator)this.m_cmp).compare(l3, l1);
        j = ((LiteralComparator)this.m_cmp).compare(l3, l2);
      }
      else if (localObject1 == Float.TYPE)
      {
        float f1 = this.m_left.getFloat(paramTuple);
        float f2 = this.m_right.getFloat(paramTuple);
        float f3 = this.m_middle.getFloat(paramTuple);
        i = ((LiteralComparator)this.m_cmp).compare(f3, f1);
        j = ((LiteralComparator)this.m_cmp).compare(f3, f2);
      }
      else if (localObject1 == Double.TYPE)
      {
        double d1 = this.m_left.getDouble(paramTuple);
        double d2 = this.m_right.getDouble(paramTuple);
        double d3 = this.m_middle.getDouble(paramTuple);
        i = ((LiteralComparator)this.m_cmp).compare(d3, d1);
        j = ((LiteralComparator)this.m_cmp).compare(d3, d2);
      }
      else
      {
        throw new IllegalStateException();
      }
    }
    else
    {
      localObject1 = this.m_left.get(paramTuple);
      Object localObject2 = this.m_right.get(paramTuple);
      Object localObject3 = this.m_middle.get(paramTuple);
      i = this.m_cmp.compare(localObject3, localObject1);
      j = this.m_cmp.compare(localObject3, localObject2);
    }
    switch (this.m_op)
    {
    case 0: 
      return (i >= 0) && (j <= 0);
    case 1: 
      return (i >= 0) && (j < 0);
    case 2: 
      return (i > 0) && (j <= 0);
    case 3: 
      return (i > 0) && (j < 0);
    }
    throw new IllegalStateException("Unknown operation.");
  }
  
  public Class getType(Schema paramSchema)
  {
    return Boolean.TYPE;
  }
  
  public Object get(Tuple paramTuple)
  {
    return getBoolean(paramTuple) ? Boolean.TRUE : Boolean.FALSE;
  }
  
  public void visit(ExpressionVisitor paramExpressionVisitor)
  {
    paramExpressionVisitor.visitExpression(this);
    paramExpressionVisitor.down();
    this.m_left.visit(paramExpressionVisitor);
    paramExpressionVisitor.up();
    paramExpressionVisitor.down();
    this.m_middle.visit(paramExpressionVisitor);
    paramExpressionVisitor.up();
    paramExpressionVisitor.down();
    this.m_right.visit(paramExpressionVisitor);
    paramExpressionVisitor.up();
  }
  
  protected void addChildListeners()
  {
    super.addChildListeners();
    this.m_middle.addExpressionListener(this);
  }
  
  protected void removeChildListeners()
  {
    super.removeChildListeners();
    this.m_middle.removeExpressionListener(this);
  }
  
  public String toString()
  {
    String str1 = "?";
    String str2 = "?";
    switch (this.m_op)
    {
    case 0: 
      str1 = str2 = "<=";
      break;
    case 1: 
      str1 = "<=";
      str2 = "<";
      break;
    case 2: 
      str1 = "<";
      str2 = "<=";
      break;
    case 3: 
      str1 = str2 = "<";
    }
    return '(' + this.m_left.toString() + ' ' + str1 + ' ' + this.m_middle.toString() + " AND " + this.m_middle.toString() + ' ' + str2 + ' ' + this.m_right.toString() + ')';
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\expression\RangePredicate.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */