package prefuse.data.io.sql;

import java.util.logging.Logger;
import prefuse.data.Table;
import prefuse.data.io.DataIOException;
import prefuse.util.PrefuseConfig;
import prefuse.util.StringLib;
import prefuse.util.collections.CopyOnWriteArrayList;

public class DataSourceWorker
  extends Thread
{
  private static Logger s_logger = Logger.getLogger(DataSourceWorker.class.getName());
  private static DataSourceWorker s_instance;
  private static CopyOnWriteArrayList s_queue;
  
  public static synchronized void submit(Entry paramEntry)
  {
    if (s_queue == null) {
      s_queue = new CopyOnWriteArrayList();
    }
    if (s_instance == null) {
      s_instance = new DataSourceWorker();
    }
    s_queue.add(paramEntry);
    synchronized (s_instance)
    {
      s_instance.notify();
    }
  }
  
  private DataSourceWorker()
  {
    super("prefuse_DatabaseWorker");
    int i = PrefuseConfig.getInt("data.io.worker.threadPriority");
    if ((i >= 1) && (i <= 10)) {
      setPriority(i);
    }
    setDaemon(true);
    start();
  }
  
  public void run()
  {
    for (;;)
    {
      Entry localEntry = null;
      synchronized (s_queue)
      {
        if (s_queue.size() > 0) {
          localEntry = (Entry)s_queue.remove(0);
        }
      }
      if (localEntry != null) {
        try
        {
          if (localEntry.listener != null) {
            localEntry.listener.preQuery(localEntry);
          }
          localEntry.ds.getData(localEntry.table, localEntry.query, localEntry.keyField, localEntry.lock);
          if (localEntry.listener != null) {
            localEntry.listener.postQuery(localEntry);
          }
        }
        catch (DataIOException ???)
        {
          s_logger.warning(((DataIOException)???).getMessage() + "\n" + StringLib.getStackTrace((Throwable)???));
        }
      } else {
        try
        {
          synchronized (this)
          {
            wait();
          }
        }
        catch (InterruptedException localInterruptedException) {}
      }
    }
  }
  
  public static abstract interface Listener
  {
    public abstract void preQuery(DataSourceWorker.Entry paramEntry);
    
    public abstract void postQuery(DataSourceWorker.Entry paramEntry);
  }
  
  public static class Entry
  {
    DatabaseDataSource ds;
    DataSourceWorker.Listener listener;
    Table table;
    String query;
    String keyField;
    Object lock;
    
    public Entry(DatabaseDataSource paramDatabaseDataSource, Table paramTable, String paramString1, String paramString2, Object paramObject, DataSourceWorker.Listener paramListener)
    {
      this.ds = paramDatabaseDataSource;
      this.table = paramTable;
      this.query = paramString1;
      this.keyField = paramString2;
      this.lock = paramObject;
      this.listener = paramListener;
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\io\sql\DataSourceWorker.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */