package prefuse.data.query;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import prefuse.data.Table;
import prefuse.data.column.ColumnMetadata;
import prefuse.data.expression.BooleanLiteral;
import prefuse.data.expression.ColumnExpression;
import prefuse.data.expression.ComparisonPredicate;
import prefuse.data.expression.Literal;
import prefuse.data.expression.OrPredicate;
import prefuse.data.expression.Predicate;
import prefuse.util.ui.JToggleGroup;

public class ListQueryBinding
  extends DynamicQueryBinding
{
  private static final String ALL = "All";
  private Class m_type;
  private ListModel m_model;
  private Listener m_lstnr;
  
  public ListQueryBinding(Table paramTable, String paramString)
  {
    super(paramTable, paramString);
    this.m_type = paramTable.getColumnType(paramString);
    this.m_lstnr = new Listener(null);
    initPredicate();
    initModel();
  }
  
  private void initPredicate()
  {
    OrPredicate localOrPredicate = new OrPredicate();
    localOrPredicate.add(BooleanLiteral.TRUE);
    setPredicate(localOrPredicate);
  }
  
  private void initModel()
  {
    if (this.m_model != null) {
      this.m_model.removeListSelectionListener(this.m_lstnr);
    }
    ColumnMetadata localColumnMetadata = this.m_table.getMetadata(this.m_field);
    Object[] arrayOfObject = localColumnMetadata.getOrdinalArray();
    this.m_model = new ListModel(arrayOfObject);
    this.m_model.addListSelectionListener(this.m_lstnr);
    this.m_model.insertElementAt("All", 0);
    this.m_model.setSelectedItem("All");
  }
  
  public JComponent createComponent()
  {
    return createCheckboxGroup();
  }
  
  public JList createList()
  {
    JList localJList = new JList(this.m_model);
    localJList.setSelectionModel(this.m_model);
    return localJList;
  }
  
  public JComboBox createComboBox()
  {
    return new JComboBox(this.m_model);
  }
  
  public JToggleGroup createCheckboxGroup()
  {
    return createToggleGroup(0);
  }
  
  public JToggleGroup createRadioGroup()
  {
    return createToggleGroup(1);
  }
  
  private JToggleGroup createToggleGroup(int paramInt)
  {
    return new JToggleGroup(paramInt, this.m_model, this.m_model);
  }
  
  private ComparisonPredicate getComparison(Object paramObject)
  {
    ColumnExpression localColumnExpression = new ColumnExpression(this.m_field);
    Literal localLiteral = Literal.getLiteral(paramObject, this.m_type);
    return new ComparisonPredicate(2, localColumnExpression, localLiteral);
  }
  
  private class Listener
    implements ListSelectionListener
  {
    private Listener() {}
    
    public void valueChanged(ListSelectionEvent paramListSelectionEvent)
    {
      ListModel localListModel = (ListModel)paramListSelectionEvent.getSource();
      OrPredicate localOrPredicate = (OrPredicate)ListQueryBinding.this.m_query;
      if (localListModel.isSelectionEmpty())
      {
        localOrPredicate.clear();
      }
      else if (localListModel.isSelectedIndex(0))
      {
        localOrPredicate.set(BooleanLiteral.TRUE);
      }
      else
      {
        int i = localListModel.getMinSelectionIndex();
        int j = localListModel.getMaxSelectionIndex();
        int k = 0;
        for (int m = i; m <= j; m++) {
          if (localListModel.isSelectedIndex(m)) {
            k++;
          }
        }
        if (k == localListModel.getSize())
        {
          localOrPredicate.set(BooleanLiteral.TRUE);
        }
        else if (k == 1)
        {
          localOrPredicate.set(ListQueryBinding.this.getComparison(localListModel.getElementAt(i)));
        }
        else
        {
          Predicate[] arrayOfPredicate = new Predicate[k];
          int n = i;
          int i1 = 0;
          while (n <= j)
          {
            if (localListModel.isSelectedIndex(n)) {
              arrayOfPredicate[(i1++)] = ListQueryBinding.this.getComparison(localListModel.getElementAt(n));
            }
            n++;
          }
          localOrPredicate.set(arrayOfPredicate);
        }
      }
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\query\ListQueryBinding.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */