package prefuse.render;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import prefuse.util.GraphicsLib;
import prefuse.visual.VisualItem;

public abstract class AbstractShapeRenderer
  implements Renderer
{
  public static final int RENDER_TYPE_NONE = 0;
  public static final int RENDER_TYPE_DRAW = 1;
  public static final int RENDER_TYPE_FILL = 2;
  public static final int RENDER_TYPE_DRAW_AND_FILL = 3;
  private int m_renderType = 3;
  protected AffineTransform m_transform = new AffineTransform();
  protected boolean m_manageBounds = true;
  
  public void setManageBounds(boolean paramBoolean)
  {
    this.m_manageBounds = paramBoolean;
  }
  
  public void render(Graphics2D paramGraphics2D, VisualItem paramVisualItem)
  {
    Shape localShape = getShape(paramVisualItem);
    if (localShape != null) {
      drawShape(paramGraphics2D, paramVisualItem, localShape);
    }
  }
  
  protected void drawShape(Graphics2D paramGraphics2D, VisualItem paramVisualItem, Shape paramShape)
  {
    GraphicsLib.paint(paramGraphics2D, paramVisualItem, paramShape, getStroke(paramVisualItem), getRenderType(paramVisualItem));
  }
  
  public Shape getShape(VisualItem paramVisualItem)
  {
    AffineTransform localAffineTransform = getTransform(paramVisualItem);
    Shape localShape = getRawShape(paramVisualItem);
    return (localAffineTransform == null) || (localShape == null) ? localShape : localAffineTransform.createTransformedShape(localShape);
  }
  
  protected BasicStroke getStroke(VisualItem paramVisualItem)
  {
    return paramVisualItem.getStroke();
  }
  
  protected abstract Shape getRawShape(VisualItem paramVisualItem);
  
  protected AffineTransform getTransform(VisualItem paramVisualItem)
  {
    return null;
  }
  
  public int getRenderType(VisualItem paramVisualItem)
  {
    return this.m_renderType;
  }
  
  public void setRenderType(int paramInt)
  {
    if ((paramInt < 0) || (paramInt > 3)) {
      throw new IllegalArgumentException("Unrecognized render type.");
    }
    this.m_renderType = paramInt;
  }
  
  public boolean locatePoint(Point2D paramPoint2D, VisualItem paramVisualItem)
  {
    if (paramVisualItem.getBounds().contains(paramPoint2D))
    {
      Shape localShape = getShape(paramVisualItem);
      return localShape != null ? localShape.contains(paramPoint2D) : false;
    }
    return false;
  }
  
  public void setBounds(VisualItem paramVisualItem)
  {
    if (!this.m_manageBounds) {
      return;
    }
    Shape localShape = getShape(paramVisualItem);
    if (localShape == null) {
      paramVisualItem.setBounds(paramVisualItem.getX(), paramVisualItem.getY(), 0.0D, 0.0D);
    } else {
      GraphicsLib.setBounds(paramVisualItem, localShape, getStroke(paramVisualItem));
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\render\AbstractShapeRenderer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */