package prefuse.util.collections;

public class IntIntTreeMap
  extends AbstractTreeMap
  implements IntIntSortedMap
{
  private IntEntry dummy = new IntEntry(Integer.MIN_VALUE, Integer.MAX_VALUE, NIL, 0);
  
  public IntIntTreeMap()
  {
    this(null, false);
  }
  
  public IntIntTreeMap(boolean paramBoolean)
  {
    this(null, paramBoolean);
  }
  
  public IntIntTreeMap(LiteralComparator paramLiteralComparator)
  {
    this(paramLiteralComparator, false);
  }
  
  public IntIntTreeMap(LiteralComparator paramLiteralComparator, boolean paramBoolean)
  {
    super(paramLiteralComparator, paramBoolean);
  }
  
  public void clear()
  {
    this.modCount += 1;
    this.size = 0;
    this.root = NIL;
  }
  
  public boolean containsKey(int paramInt)
  {
    return find(paramInt, 0) != NIL;
  }
  
  public int get(int paramInt)
  {
    AbstractTreeMap.Entry localEntry = find(paramInt, 0);
    return localEntry == NIL ? Integer.MIN_VALUE : localEntry.val;
  }
  
  public int put(int paramInt1, int paramInt2)
  {
    AbstractTreeMap.Entry localEntry = this.root;
    this.lastOrder = 0;
    if (localEntry == NIL)
    {
      incrementSize(true);
      this.root = new IntEntry(paramInt1, paramInt2, NIL, this.lastOrder);
      return Integer.MIN_VALUE;
    }
    this.dummy.key = paramInt1;
    this.dummy.order = Integer.MAX_VALUE;
    for (;;)
    {
      int i = compare(this.dummy, localEntry);
      if (i == 0) {
        return localEntry.setValue(paramInt2);
      }
      if (i < 0)
      {
        if (localEntry.left != NIL)
        {
          localEntry = localEntry.left;
        }
        else
        {
          incrementSize(this.lastOrder == 0);
          localEntry.left = new IntEntry(paramInt1, paramInt2, localEntry, this.lastOrder);
          fixUpInsert(localEntry.left);
          return Integer.MIN_VALUE;
        }
      }
      else if (localEntry.right != NIL)
      {
        localEntry = localEntry.right;
      }
      else
      {
        incrementSize(this.lastOrder == 0);
        localEntry.right = new IntEntry(paramInt1, paramInt2, localEntry, this.lastOrder);
        fixUpInsert(localEntry.right);
        return Integer.MIN_VALUE;
      }
    }
  }
  
  public int remove(int paramInt)
  {
    AbstractTreeMap.Entry localEntry;
    if (this.allowDuplicates) {
      localEntry = findPredecessor(paramInt, Integer.MAX_VALUE);
    } else {
      localEntry = find(paramInt, 0);
    }
    if (localEntry == NIL) {
      return Integer.MIN_VALUE;
    }
    int i = localEntry.val;
    remove(localEntry);
    return i;
  }
  
  public int remove(int paramInt1, int paramInt2)
  {
    AbstractTreeMap.Entry localEntry = findCeiling(paramInt1, 0);
    if ((localEntry != NIL) && (localEntry.getIntKey() != paramInt1)) {
      localEntry = successor(localEntry);
    }
    if ((localEntry == NIL) || (localEntry.getIntKey() != paramInt1)) {
      return Integer.MIN_VALUE;
    }
    while ((localEntry.val != paramInt2) && (localEntry != NIL)) {
      localEntry = successor(localEntry);
    }
    if (localEntry == NIL) {
      return Integer.MIN_VALUE;
    }
    remove(localEntry);
    return paramInt2;
  }
  
  public int getLast(int paramInt)
  {
    AbstractTreeMap.Entry localEntry = findPredecessor(paramInt, Integer.MAX_VALUE);
    return (localEntry == NIL) || (((IntEntry)localEntry).key != paramInt) ? Integer.MIN_VALUE : localEntry.val;
  }
  
  public int getPreviousValue(int paramInt1, int paramInt2)
  {
    AbstractTreeMap.Entry localEntry = find(paramInt1, paramInt2);
    return predecessor(localEntry).val;
  }
  
  public int getNextValue(int paramInt1, int paramInt2)
  {
    AbstractTreeMap.Entry localEntry = find(paramInt1, paramInt2);
    return successor(localEntry).val;
  }
  
  public int firstKey()
  {
    return minimum(this.root).getIntKey();
  }
  
  public int lastKey()
  {
    return maximum(this.root).getIntKey();
  }
  
  public LiteralIterator keyIterator()
  {
    return new KeyIterator();
  }
  
  public LiteralIterator keyRangeIterator(int paramInt1, boolean paramBoolean1, int paramInt2, boolean paramBoolean2)
  {
    AbstractTreeMap.Entry localEntry1;
    AbstractTreeMap.Entry localEntry2;
    if (this.cmp.compare(paramInt1, paramInt2) <= 0)
    {
      localEntry1 = findCeiling(paramInt1, paramBoolean1 ? 0 : Integer.MAX_VALUE);
      localEntry2 = findCeiling(paramInt2, paramBoolean2 ? Integer.MAX_VALUE : 0);
    }
    else
    {
      localEntry1 = findCeiling(paramInt1, paramBoolean1 ? Integer.MAX_VALUE : 0);
      localEntry1 = predecessor(localEntry1);
      localEntry2 = findCeiling(paramInt2, paramBoolean2 ? 0 : Integer.MAX_VALUE);
      localEntry2 = predecessor(localEntry2);
    }
    return new KeyIterator(localEntry1, localEntry2);
  }
  
  public IntIterator valueRangeIterator(int paramInt1, boolean paramBoolean1, int paramInt2, boolean paramBoolean2)
  {
    return new AbstractTreeMap.ValueIterator(this, (AbstractTreeMap.EntryIterator)keyRangeIterator(paramInt1, paramBoolean1, paramInt2, paramBoolean2));
  }
  
  protected int compare(AbstractTreeMap.Entry paramEntry1, AbstractTreeMap.Entry paramEntry2)
  {
    int i = this.cmp.compare(paramEntry1.getIntKey(), paramEntry2.getIntKey());
    if ((this.allowDuplicates) && (i == 0))
    {
      i = paramEntry1.order > paramEntry2.order ? 1 : paramEntry1.order < paramEntry2.order ? -1 : 0;
      this.lastOrder = (1 + (i < 0 ? paramEntry1.order : paramEntry2.order));
    }
    return i;
  }
  
  private AbstractTreeMap.Entry find(int paramInt1, int paramInt2)
  {
    this.dummy.key = paramInt1;
    this.dummy.order = paramInt2;
    AbstractTreeMap.Entry localEntry = find(this.dummy);
    return localEntry;
  }
  
  private AbstractTreeMap.Entry findPredecessor(int paramInt1, int paramInt2)
  {
    this.dummy.key = paramInt1;
    this.dummy.order = paramInt2;
    AbstractTreeMap.Entry localEntry = findPredecessor(this.dummy);
    return localEntry;
  }
  
  private AbstractTreeMap.Entry findCeiling(int paramInt1, int paramInt2)
  {
    this.dummy.key = paramInt1;
    this.dummy.order = paramInt2;
    AbstractTreeMap.Entry localEntry = findCeiling(this.dummy);
    return localEntry;
  }
  
  private class KeyIterator
    extends AbstractTreeMap.KeyIterator
  {
    public KeyIterator()
    {
      super();
    }
    
    public KeyIterator(AbstractTreeMap.Entry paramEntry1, AbstractTreeMap.Entry paramEntry2)
    {
      super(paramEntry1, paramEntry2);
    }
    
    public boolean isIntSupported()
    {
      return true;
    }
    
    public int nextInt()
    {
      return nextEntry().getIntKey();
    }
  }
  
  static class IntEntry
    extends AbstractTreeMap.Entry
  {
    int key;
    
    public IntEntry(int paramInt1, int paramInt2)
    {
      super();
      this.key = paramInt1;
    }
    
    public IntEntry(int paramInt1, int paramInt2, AbstractTreeMap.Entry paramEntry, int paramInt3)
    {
      super(paramEntry, paramInt3);
      this.key = paramInt1;
    }
    
    public int getIntKey()
    {
      return this.key;
    }
    
    public Object getKey()
    {
      return new Integer(this.key);
    }
    
    public boolean keyEquals(AbstractTreeMap.Entry paramEntry)
    {
      return ((paramEntry instanceof IntEntry)) && (this.key == ((IntEntry)paramEntry).key);
    }
    
    public boolean equals(Object paramObject)
    {
      if (!(paramObject instanceof IntEntry)) {
        return false;
      }
      IntEntry localIntEntry = (IntEntry)paramObject;
      return (this.key == localIntEntry.key) && (this.val == localIntEntry.val);
    }
    
    public int hashCode()
    {
      int i = this.key;
      int j = this.val;
      return i ^ j ^ this.order;
    }
    
    public String toString()
    {
      return this.key + "=" + this.val;
    }
    
    public void copyFields(AbstractTreeMap.Entry paramEntry)
    {
      super.copyFields(paramEntry);
      this.key = paramEntry.getIntKey();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\collections\IntIntTreeMap.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */