package prefuse.util.collections;

public class LongIntTreeMap
  extends AbstractTreeMap
  implements LongIntSortedMap
{
  private LongEntry dummy = new LongEntry(Long.MIN_VALUE, Integer.MAX_VALUE, NIL, 0);
  
  public LongIntTreeMap()
  {
    this(null, false);
  }
  
  public LongIntTreeMap(boolean paramBoolean)
  {
    this(null, paramBoolean);
  }
  
  public LongIntTreeMap(LiteralComparator paramLiteralComparator)
  {
    this(paramLiteralComparator, false);
  }
  
  public LongIntTreeMap(LiteralComparator paramLiteralComparator, boolean paramBoolean)
  {
    super(paramLiteralComparator, paramBoolean);
  }
  
  public void clear()
  {
    this.modCount += 1;
    this.size = 0;
    this.root = NIL;
  }
  
  public boolean containsKey(long paramLong)
  {
    return find(paramLong, 0) != NIL;
  }
  
  public int get(long paramLong)
  {
    AbstractTreeMap.Entry localEntry = find(paramLong, 0);
    return localEntry == NIL ? Integer.MIN_VALUE : localEntry.val;
  }
  
  public int put(long paramLong, int paramInt)
  {
    AbstractTreeMap.Entry localEntry = this.root;
    this.lastOrder = 0;
    if (localEntry == NIL)
    {
      incrementSize(true);
      this.root = new LongEntry(paramLong, paramInt, NIL, this.lastOrder);
      return Integer.MIN_VALUE;
    }
    this.dummy.key = paramLong;
    this.dummy.order = Integer.MAX_VALUE;
    for (;;)
    {
      int i = compare(this.dummy, localEntry);
      if (i == 0) {
        return localEntry.setValue(paramInt);
      }
      if (i < 0)
      {
        if (localEntry.left != NIL)
        {
          localEntry = localEntry.left;
        }
        else
        {
          incrementSize(this.lastOrder == 0);
          localEntry.left = new LongEntry(paramLong, paramInt, localEntry, this.lastOrder);
          fixUpInsert(localEntry.left);
          return Integer.MIN_VALUE;
        }
      }
      else if (localEntry.right != NIL)
      {
        localEntry = localEntry.right;
      }
      else
      {
        incrementSize(this.lastOrder == 0);
        localEntry.right = new LongEntry(paramLong, paramInt, localEntry, this.lastOrder);
        fixUpInsert(localEntry.right);
        return Integer.MIN_VALUE;
      }
    }
  }
  
  public int remove(long paramLong)
  {
    AbstractTreeMap.Entry localEntry;
    if (this.allowDuplicates) {
      localEntry = findPredecessor(paramLong, Integer.MAX_VALUE);
    } else {
      localEntry = find(paramLong, 0);
    }
    if (localEntry == NIL) {
      return Integer.MIN_VALUE;
    }
    int i = localEntry.val;
    remove(localEntry);
    return i;
  }
  
  public int remove(long paramLong, int paramInt)
  {
    AbstractTreeMap.Entry localEntry = findCeiling(paramLong, 0);
    if ((localEntry != NIL) && (localEntry.getLongKey() != paramLong)) {
      localEntry = successor(localEntry);
    }
    if ((localEntry == NIL) || (localEntry.getLongKey() != paramLong)) {
      return Integer.MIN_VALUE;
    }
    while ((localEntry.val != paramInt) && (localEntry != NIL)) {
      localEntry = successor(localEntry);
    }
    if (localEntry == NIL) {
      return Integer.MIN_VALUE;
    }
    remove(localEntry);
    return paramInt;
  }
  
  public long firstKey()
  {
    return minimum(this.root).getLongKey();
  }
  
  public long lastKey()
  {
    return maximum(this.root).getLongKey();
  }
  
  public LiteralIterator keyIterator()
  {
    return new KeyIterator();
  }
  
  public LiteralIterator keyRangeIterator(long paramLong1, boolean paramBoolean1, long paramLong2, boolean paramBoolean2)
  {
    AbstractTreeMap.Entry localEntry1;
    AbstractTreeMap.Entry localEntry2;
    if (this.cmp.compare(paramLong1, paramLong2) <= 0)
    {
      localEntry1 = findCeiling(paramLong1, paramBoolean1 ? 0 : Integer.MAX_VALUE);
      localEntry2 = findCeiling(paramLong2, paramBoolean2 ? Integer.MAX_VALUE : 0);
    }
    else
    {
      localEntry1 = findCeiling(paramLong1, paramBoolean1 ? Integer.MAX_VALUE : 0);
      localEntry1 = predecessor(localEntry1);
      localEntry2 = findCeiling(paramLong2, paramBoolean2 ? 0 : Integer.MAX_VALUE);
      localEntry2 = predecessor(localEntry2);
    }
    return new KeyIterator(localEntry1, localEntry2);
  }
  
  public IntIterator valueRangeIterator(long paramLong1, boolean paramBoolean1, long paramLong2, boolean paramBoolean2)
  {
    return new AbstractTreeMap.ValueIterator(this, (AbstractTreeMap.EntryIterator)keyRangeIterator(paramLong1, paramBoolean1, paramLong2, paramBoolean2));
  }
  
  protected int compare(AbstractTreeMap.Entry paramEntry1, AbstractTreeMap.Entry paramEntry2)
  {
    int i = this.cmp.compare(paramEntry1.getLongKey(), paramEntry2.getLongKey());
    if ((this.allowDuplicates) && (i == 0))
    {
      i = paramEntry1.order > paramEntry2.order ? 1 : paramEntry1.order < paramEntry2.order ? -1 : 0;
      this.lastOrder = (1 + (i < 0 ? paramEntry1.order : paramEntry2.order));
    }
    return i;
  }
  
  private AbstractTreeMap.Entry find(long paramLong, int paramInt)
  {
    this.dummy.key = paramLong;
    this.dummy.order = paramInt;
    AbstractTreeMap.Entry localEntry = find(this.dummy);
    return localEntry;
  }
  
  private AbstractTreeMap.Entry findPredecessor(long paramLong, int paramInt)
  {
    this.dummy.key = paramLong;
    this.dummy.order = paramInt;
    AbstractTreeMap.Entry localEntry = findPredecessor(this.dummy);
    return localEntry;
  }
  
  private AbstractTreeMap.Entry findCeiling(long paramLong, int paramInt)
  {
    this.dummy.key = paramLong;
    this.dummy.order = paramInt;
    AbstractTreeMap.Entry localEntry = findCeiling(this.dummy);
    return localEntry;
  }
  
  private class KeyIterator
    extends AbstractTreeMap.KeyIterator
  {
    public KeyIterator()
    {
      super();
    }
    
    public KeyIterator(AbstractTreeMap.Entry paramEntry1, AbstractTreeMap.Entry paramEntry2)
    {
      super(paramEntry1, paramEntry2);
    }
    
    public boolean isLongSupported()
    {
      return true;
    }
    
    public long nextLong()
    {
      return nextEntry().getLongKey();
    }
  }
  
  static class LongEntry
    extends AbstractTreeMap.Entry
  {
    long key;
    
    public LongEntry(long paramLong, int paramInt)
    {
      super();
      this.key = paramLong;
    }
    
    public LongEntry(long paramLong, int paramInt1, AbstractTreeMap.Entry paramEntry, int paramInt2)
    {
      super(paramEntry, paramInt2);
      this.key = paramLong;
    }
    
    public long getLongKey()
    {
      return this.key;
    }
    
    public Object getKey()
    {
      return new Long(this.key);
    }
    
    public boolean keyEquals(AbstractTreeMap.Entry paramEntry)
    {
      return ((paramEntry instanceof LongEntry)) && (this.key == ((LongEntry)paramEntry).key);
    }
    
    public boolean equals(Object paramObject)
    {
      if (!(paramObject instanceof LongEntry)) {
        return false;
      }
      LongEntry localLongEntry = (LongEntry)paramObject;
      return (this.key == localLongEntry.key) && (this.val == localLongEntry.val);
    }
    
    public int hashCode()
    {
      int i = (int)(this.key ^ this.key >>> 32);
      int j = this.val;
      return i ^ j ^ this.order;
    }
    
    public String toString()
    {
      return this.key + "=" + this.val;
    }
    
    public void copyFields(AbstractTreeMap.Entry paramEntry)
    {
      super.copyFields(paramEntry);
      this.key = paramEntry.getLongKey();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\collections\LongIntTreeMap.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */